#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QDebug>
#include <QMessageBox>
#include <QWebFrame>
#include <QSettings>
#include <QFile>
#include <QDate>
#include <QMap>

#include <QProcess>
#include <QCryptographicHash>
#include <QRegExp>

#include <unistd.h>
#include <utmp.h>
#include <QUrl>
#include <QDir>
#include <QTextCodec>
#include <QNetworkInterface>

#define CACHE_FILE "/home/%1/.config/fcitx/fcitx-uc.cache"
#define WEB_CONFIG_FILE "/usr/share/fcitx/personal-center/web_config"
#define USER_PROFILE "/usr/share/fcitx/personal-center/fcitx_user" //ini format file

int onLoadAll_flag = 0;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    QDir::setCurrent("/usr/share/fcitx/personal-center");
    su = NULL;
    this->setWindowFlags(this->windowFlags() & ~Qt::WindowMaximizeButtonHint);
    this->setWindowIcon(QIcon("/usr/share/icons/hicolor/scalable/apps/fcitx.svg"));
    setFixedSize(600, 443);
    ui->setupUi(this);

    connect(ui->webView->page()->mainFrame(), SIGNAL(javaScriptWindowObjectCleared()), this, SLOT(attachQtObject()));
    connect(ui->webView, SIGNAL(loadStarted()), this, SLOT(slot_loadStarted()));
    connect(ui->webView, SIGNAL(loadFinished(bool)), this, SLOT(slot_loadFinished(bool)));

    email = validateUserInfo();
    if(email == NULL) {
        QMessageBox message(QMessageBox::Information, tr("info"), "用户信息不存在或已损坏，请重新下载并安装授权文件！", QMessageBox::Ok,NULL);
        message.setButtonText (QMessageBox::Ok, tr("OK"));
        message.exec();
        exit(0);
    }

    //cacheFile = QString(CACHE_FILE).arg(getCurrentLoginUser());
    QString w_file = getFileContent(WEB_CONFIG_FILE);
    URL = QString("http://%1").arg(w_file);
    qDebug()<<URL<<email<<this->getFcitxVersion();
    //loadLoginCache();
    QString url = QString("%1/im_service/user_linux/index.action?email=%2&os=l&ver=%3").arg(URL).arg(email).arg(getFcitxVersion());
    ui->webView->setUrl(QUrl(url));
}

MainWindow::~MainWindow()
{
    delete ui;
}

QString MainWindow::getFcitxVersion() {
    QString v = "";
    QProcess process;
    process.start("dpkg", QStringList()<<"-s"<<"fcitx");
    process.waitForFinished();
    char data[1024];
    //Version: 1:4.2.8.3-3  ==> 4.2.8.3-3
    while(-1 != process.readLine(data, 1024)) {
        QString str = QString(data);
        if(str.startsWith("Version:")) {
            str = str.trimmed();
            v = str.mid(str.lastIndexOf(":") + 1);
            //qDebug()<<v.trimmed();
            break;
        }
    }
    process.close();
    v = QUrl::toPercentEncoding(v.toUtf8());//to encodeing
    //qDebug()<<v;
    return v;
}

void MainWindow::loadLoginCache()
{
    cookie = new QNetworkCookieJar();
    ui->webView->page()->networkAccessManager()->setCookieJar(cookie);
    QList<QNetworkCookie> cl;

    if(QFile::exists(cacheFile)) {
        cl.append(QNetworkCookie(QByteArray("JSESSIONID"), getLoginSession().toAscii()));
        cl.append(QNetworkCookie(QByteArray("_USER_"), getLastUser().toAscii()));

        QSettings settings(cacheFile, QSettings::IniFormat);
        settings.beginGroup("pwd");
        QStringList keys = settings.allKeys();
        for(int i = 0;i < keys.length();i++) {
            QString k = keys.at(i);
            QString v = settings.value(k).toString();
            qDebug()<<k<<v;
            cl.append(QNetworkCookie(k.toAscii(), v.toAscii()));
        }
    }

    cookie->setCookiesFromUrl(cl, QUrl(URL));
}

QString MainWindow::validateUserInfo()
{
    QString email64 = getRegVal(USER_PROFILE, "User/EM");
    QString md5 = getRegVal(USER_PROFILE, "User/MD");
    if(email64 == "" || md5 == "") return NULL;
    QString email = QString(QByteArray::fromBase64(email64.toLatin1()));
    QByteArray arr = QCryptographicHash::hash (email64.toLatin1(), QCryptographicHash::Md5);
    QString tar_md5(arr.toHex());
    QRegExp re("^[\\w-]+(\\.[\\w-]+)*@[\\w-]+(\\.[\\w-]+)+$");
    if(md5.toLower() == tar_md5.toLower() && re.exactMatch(email)) {
        return email;
    }
    return NULL;
}

void MainWindow::attachQtObject()
{
    ui->webView->page()->mainFrame()->addToJavaScriptWindowObject("mw", this);
}

void MainWindow::msgBox(QString title, QString info)
{
    QMessageBox message(QMessageBox::Information, title, info, QMessageBox::Ok,NULL);
    message.setButtonText (QMessageBox::Ok, tr("OK"));
    if (message.exec() == QMessageBox::Ok) {
        closeWindow();
    }
}

void MainWindow::saveLoginSession(QString session)
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    settings.beginGroup("user");
    settings.setValue("sess", session);
    settings.endGroup();
}

void MainWindow::removeLoginSession()
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    settings.remove("user/sess");
}

QString MainWindow::getLastUser()
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    QString last = settings.value("user/last").toString();
    return last;
}

void MainWindow::saveUid(QString uid, QString uName)
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    settings.beginGroup("user");
    settings.setValue("uid", uid);
    settings.setValue("uName", uName);
    settings.endGroup();
}

void MainWindow::removeUid()
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    settings.remove("user/uid");
    settings.remove("user/uName");
}

QString MainWindow::getPwd(QString user)
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    QString last = settings.value(QString("pwd/%1").arg(user)).toString();
    return last;
}

void MainWindow::savePwd(QString user, QString pwd)
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    settings.beginGroup("pwd");
    settings.setValue(user, pwd);
    settings.endGroup();
}

void MainWindow::removePwd(QString user)
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    settings.remove(QString("pwd/%1").arg(user));
}

QString MainWindow::getLoginSession()
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    QString sess = settings.value("user/sess").toString();
    return sess;
}

void MainWindow::saveLastUser(QString user)
{
    QSettings settings(cacheFile, QSettings::IniFormat);
    settings.beginGroup("user");
    settings.setValue("last", user);
    settings.endGroup();
}

void MainWindow::slot_loadStarted()
{

}

void MainWindow::slot_loadFinished(bool ok)
{
    if(!ok) {
        msgBox(tr("info"), tr("Failed to load the page, please try again later or check whether the network is connect!"));
    }
}

void MainWindow::closeEvent(QCloseEvent *evt)
{
    ui->webView->page()->mainFrame()->evaluateJavaScript("onClose()");
    evt->accept();
}

QString MainWindow::getCurrentLoginUser() {
    char* user = new char[512];
    strcpy(user, "");
    struct utmp record;
    FILE *fp;
    int strulen = sizeof(record);
    if((fp=fopen("/var/run/utmp", "r"))==0){
        perror("fopen");
        return NULL;
    }
    memset(&record, 0x00, strulen);
    time_t cur = time(NULL);
    while(fread(&record,strulen,1,fp)){
        if(record.ut_type == USER_PROCESS
                && strcmp(record.ut_host,":0") == 0
                && (strcmp(record.ut_line, "tty7") == 0 || strcmp(record.ut_line, "tty8") == 0)) {
            ////cout<<record.ut_user<<"----"<<record.ut_host<<"\t"<<record.ut_line<<endl;
            if(record.ut_tv.tv_sec < cur) {
                cur = record.ut_tv.tv_sec;
                strcpy(user, record.ut_user);
                break;
            }
        }
    }
    fclose(fp);
    ////cout<<"----"<<user<<"\t"<<cur<<endl;
    return QString(user);
}

QString MainWindow::getRegVal(QString path, QString key, QString default_val) {
    QSettings *st = new QSettings(path, QSettings::IniFormat);
    QString rs = st->value(key, default_val).toString();
    delete st;
    return rs;
}

void MainWindow::closeWindow()
{
    //qDebug()<<"exit app!";
    this->close();
    //QApplication::exit();
}

QString MainWindow::getFileContent(QString filePathName)
{
    QString fileContent="";
    QFile tempfile(filePathName);
    if(!tempfile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug()<<"Can't open the file!"<<filePathName;
        fileContent = "0";
    }
    QTextStream txtInput(&tempfile);
    QString lineStr;
    while(!txtInput.atEnd()) {
        lineStr = txtInput.readLine();
        //qDebug()<<lineStr;
        fileContent += lineStr;
    }

    tempfile.close();
    //qDebug()<<"current local fileContent is :"<<fileContent;
    return fileContent;
}

///////////////////////////////////////////////////////////////////////////////////
void MainWindow::onLoadedAll()//页面初始化完成回调
{
    qDebug()<<"Loaded Page Complete!";
    init();
    /*this->setCurSpeedVal(123);
    this->setRecordSpeedVal(125);
    */
}

void MainWindow::setCumVal(long cum)
{
    this->cumCache = cum;
    QString script = QString("setCumVal(%1)").arg(QString::number(cum));
    ui->webView->page()->mainFrame()->evaluateJavaScript(script);
}

long MainWindow::getCumVal()
{
    return this->cumCache;
}

void MainWindow::setRecordSpeedVal(int speed)
{
    this->recordSpeedCache = speed;
    QString script = QString("setRecordSpeedVal(%1)").arg(QString::number(speed));
    ui->webView->page()->mainFrame()->evaluateJavaScript(script);
}

int MainWindow::getRecordSpeedVal()//get last set cache
{
    return this->recordSpeedCache;
}

void MainWindow::setCurSpeedVal(int speed)
{
    this->speedCache = speed;
    QString script = QString("setCurSpeedVal(%1)").arg(QString::number(speed));
    ui->webView->page()->mainFrame()->evaluateJavaScript(script);
}

int MainWindow::getCurSpeedVal()//get last set cache
{
    return this->speedCache;
}

void MainWindow::setTodayVal(long val)
{
    QString mac=getLocalMac();
    QString mac_md5=md5_encode(mac);
    this->todayCache = val;
   // QString script = QString("setTodayVal(%1,%2)").arg(QString::number(val),mac_md5);
    QString script = QString("setTodayVal(%1)").arg(QString::number(val));
    ui->webView->page()->mainFrame()->evaluateJavaScript(script);
}
void MainWindow::setMacmd5()
{
    QString mac=getLocalMac();
    QString mac_md5=md5_encode(mac);
   // QString script = QString("setTodayVal(%1,%2)").arg(QString::number(val),mac_md5);
    QString script = QString("setMacmd5(%1)").arg(mac_md5);
    ui->webView->page()->mainFrame()->evaluateJavaScript(script);
}
long MainWindow::getTodayVal()
{
    return this->todayCache;
}

void MainWindow::serverdealMessage(QString m_server_message)
{
    qDebug()<<"-------------进入处理客户端函数 ---------";
    qDebug()<<onLoadAll_flag;
    qDebug()<<m_server_message;
    if (onLoadAll_flag == 0)
        return;
    QString current_input_count;
    QString total_input_count;
    QString current_input_speed;
    QString record_input_speed;

//    qDebug()<<"receive Message is:"<<m_server_message;
    current_input_count = m_server_message.section(',',0,0);
 //   qDebug()<<"first value is"<<current_input_count;
    total_input_count = m_server_message.section(',',1,1);
 //   qDebug()<<"second Message is:" <<total_input_count;
    current_input_speed = m_server_message.section(',',2,2);
 //   qDebug()<<"third Message is :" << current_input_speed;
    record_input_speed = m_server_message.section(',',3,3);
 //   qDebug()<<"fourth Message is :" <<record_input_speed;
    ///设置输入总数
    setCumVal(total_input_count.toLong());
    ///设置当前输入字数
    setTodayVal(current_input_count.toLong());
    ///设置输入速度
    setCurSpeedVal(current_input_speed.toInt());
    ///设置最大输入速度
    setRecordSpeedVal(record_input_speed.toInt());
}

void MainWindow::init()
{
    qDebug()<<"init";
    QVariant val = ui->webView->page()->mainFrame()->evaluateJavaScript("getCurrentUser()");
    QString login_username = val.toString();
    qDebug()<< "login_username is "<<login_username;
    qDebug()<<"system log name is "<<getCurrentLoginUser();

    QString conf_dir = "/home/"+getCurrentLoginUser()+"/.config/fcitx/pinyin/";

    QString init_today_input_num = getFileContent(conf_dir + "inputHZcoutToday-"+login_username );
    qDebug()<< "init_today_input_num=" << init_today_input_num;

    QString init_total_input_num = getFileContent(conf_dir + "inputHZcountTotal-"+login_username);
    qDebug() << "init_total_input_num=" << init_total_input_num;

    QString init_record_input_speed = getFileContent(conf_dir + "inputHZcountspeedrecord-" + login_username );
    qDebug() << "init_record_input_speed=" << init_record_input_speed << endl;

    ///设置输入总数
    setCumVal(init_total_input_num.toLong());
    ///设置当前输入字数
    setTodayVal(init_today_input_num.toLong());
    ///设置输入速度
    //setCurSpeedVal(init_cur_input_speed.toInt());
    setCurSpeedVal(0);
    ///设置最大输入速度
    setRecordSpeedVal(init_record_input_speed.toInt());

    /*QString user_dir = getCurrentUserAPPData();

    QDateTime time = QDateTime::currentDateTime();
    QString curDate = time.toString("yyyy-MM-dd");
    qDebug() << "今日文件名称" <<curDate;
    qDebug()<< "getCurrentUserAPPData=" << user_dir;

    QString init_today_input_num = getFileContent(user_dir + "\\LoginUser\\" + login_username + "\\" + curDate );
    qDebug()<< "init_today_input_num=" << init_today_input_num;

    QString init_total_input_num = getFileContent(user_dir + "\\LoginUser\\" + login_username + "\\total_input");
//    qDebug() << "init_total_input_num=" << init_total_input_num;

    QString init_cur_input_speed = getFileContent(user_dir + "\\LoginUser\\" + login_username + "\\speed");
 //   qDebug() << "init_cur_input_speed=" << init_cur_input_speed << endl;

    QString init_record_input_speed = getFileContent(user_dir + "\\LoginUser\\" + login_username + "\\speedrecord");
    qDebug() << "init_record_input_speed=" << init_record_input_speed;

    ///设置输入总数
    setCumVal(init_total_input_num.toLong());
    ///设置当前输入字数
    setTodayVal(init_today_input_num.toLong());
    ///设置输入速度
    //setCurSpeedVal(init_cur_input_speed.toInt());
    setCurSpeedVal(0);
    ///设置最大输入速度
    setRecordSpeedVal(init_record_input_speed.toInt());*/

    onLoadAll_flag = 1;
}

void MainWindow :: writefile(QString sFilePath,QString m_content)
{
    QFile file(sFilePath);
    //方式：Append为追加，WriteOnly，ReadOnly
    if (!file.open(QIODevice::WriteOnly|QIODevice::Text)) {
        QMessageBox::critical(NULL, "提示", "无法创建文件");
        return;
    }
    QTextStream out(&file);
    out<<m_content<<endl;
    out.flush();
    file.close();
}
void MainWindow::changefile(QString sFilePath,QString m_content,QString origin)
{
    qDebug()<<"sddsssssssssssssssss"<<sFilePath;
    QDate time=QDate::currentDate();
    QString year=QString::number(time.year(), 10);
    QString month=QString::number(time.month(),10);
    QString day=QString::number(time.day(),10);
    QString compact_string=year+"year"+month+"month"+day+"day:";
    qDebug()<<"sssssssssssssss"<<compact_string;


    QString strAll;
     QStringList strList;
     QFile readFile(sFilePath);
     if(readFile.open((QIODevice::ReadOnly|QIODevice::Text)))
     {
         QTextStream stream(&readFile);
         strAll=stream.readAll();
     }
     readFile.close();
     QFile writeFile(sFilePath);
     if(writeFile.open(QIODevice::WriteOnly|QIODevice::Text))
     {
             QTextStream stream(&writeFile);
             strList=strAll.split("\n");
             for(int i=0;i<strList.count();i++)
             {
                 if(strList.at(i).contains(compact_string))
                 {
                     if(i==strList.count()-1)
                     {
                        stream<<compact_string<<m_content;
                     }
                     else
                     {
                         stream<<compact_string<<m_content<<'\n';
                     }

                 }
                 else
                 {
                 if(i==strList.count()-1)
                 {
                     //最后一行不需要换行
                     stream<<strList.at(i);
                 }
                 else
                 {
                     stream<<strList.at(i)<<'\n';
                 }
                 }


             }
     }
     writeFile.close();
}

QString  MainWindow::getLocalMac()
{
    QList<QNetworkInterface> nets = QNetworkInterface::allInterfaces();
    QNetworkInterface netInterface;
    QString mac="";
    foreach (netInterface, nets)
    {
        mac=netInterface.hardwareAddress();
    }
    return mac;
}
QString MainWindow::md5_encode(QString m_in)
{
    QByteArray ba;
    ba =QCryptographicHash::hash(m_in.toLatin1(),QCryptographicHash::Md5);
    QString tempFilenamemd5(ba.toHex());
    return tempFilenamemd5;
}

void MainWindow::syncUserDB(QString email)
{
    qDebug()<<"Sync User Data Base!"<<email;//email参数暂时没有使用
    //同步词库（异步）下载词库,导入词库,导出词库,上传词库
    if(su == NULL) {
        su = new SyncUserDB();
        su->connect(su, SIGNAL(completeSync(bool, int)), this, SLOT(slot_complete_sync(bool, int)), Qt::UniqueConnection);
    }
    if(su->init()) {//同步之前必须调用init检测并初始化用户及相关数据
        su->sync();
    }
    syncTotalinputNum(email);
    syncTodayTotalinputNum(email);

}
void MainWindow::syncTodayTotalinputNum(QString email)
{
    QString mac=getLocalMac();
    QDate datetime=QDate::currentDate();
    QString active_time=datetime.toString("yyyy-MM-dd");
    QNetworkAccessManager *manager= new QNetworkAccessManager(this);
    connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(TodayreplyFinished(QNetworkReply*)));
    QNetworkRequest *request = new QNetworkRequest();
    request->setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");

    QString test ="http://192.168.0.101:8080/im_service/user_ex/queryTodayWordCumu.action?";
  //  qDebug()<< "请求链接为："<<test;
    QString w_file = getFileContent(QString(WEB_CONFIG_FILE));
    URL = QString("http://%1").arg(w_file);
    QString url = QString("%1/im_service/user_ex/queryTodayWordCumu.action?").arg(URL);
    request->setUrl(QUrl(url));
 // request->setUrl(QUrl(test));
    QByteArray postData;
    postData.append("os=l");
    postData.append("&email=");
    postData.append(email);
    postData.append("&mac=");
    postData.append(mac);
    postData.append("&datetime=");
    postData.append(active_time);
    QNetworkReply* reply = manager->post(*request, postData);
    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(upLoadError(QNetworkReply::NetworkError)));
    connect(reply, SIGNAL(uploadProgress(qint64, qint64 )), this, SLOT(OnUploadProgress(qint64, qint64)));
}
void MainWindow::TodayreplyFinished(QNetworkReply *reply)
{
    if(reply->error() == QNetworkReply::NoError)
    {
        qDebug()<< "-------------处理没有出错,开始同步个人中心输入总数---------";
        QByteArray bytes = reply->readAll();  //获取字节
        qDebug()<< "byte is "<<bytes;
        QString result(bytes);  //转化为字符串
        qDebug()<<result;

        QString user_today_filepath="/home/"+getCurrentLoginUser()+"/.config/fcitx/pinyin/inputHZcoutToday-"+email;
        QString user_conut_filepath="/home/"+getCurrentLoginUser()+"/.config/fcitx/pinyin/inputHZcount-"+email;

        qDebug()<< "文件路径为："<<user_today_filepath;

         QDir dir;
//        if(!dir.exists(user_total_dirpath))//判断需要创建的文件夹是否存在
//        {
//            dir.mkdir(user_total_dirpath); //创建文件夹
//        }


        qDebug()<<"本地文件记录的总数为："<<getFileContent(user_today_filepath).toInt();
        if (getFileContent(user_today_filepath).toInt()<result.toInt())
        {
            qDebug()<<"开始写文件";
            changefile(user_conut_filepath,result,getFileContent(user_today_filepath));

            writefile(user_today_filepath,result);
            setTodayVal(result.toInt());
        }
 //       writefile(user_total_filepath,result);

    }
    else
    {
        //处理错误
        qDebug()<<"Error" ;

    }
}
void MainWindow::syncTotalinputNum(QString email)
{
    QNetworkAccessManager *manager= new QNetworkAccessManager(this);
    connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(replyFinished(QNetworkReply*)));
    QNetworkRequest *request = new QNetworkRequest();
    request->setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");

   // QString test ="http://192.168.1.106:8080/im_service/user_ex/queryWordCumu.action?";
  //  qDebug()<< "请求链接为："<<test;
    QString w_file = getFileContent(QString(WEB_CONFIG_FILE));
    URL = QString("http://%1").arg(w_file);
    QString url = QString("%1/im_service/user_ex/queryWordCumu.action?").arg(URL);
    request->setUrl(QUrl(url));
    QByteArray postData;
    postData.append("os=l");
    postData.append("&email=");
    postData.append(email);
    QNetworkReply* reply = manager->post(*request, postData);
    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(upLoadError(QNetworkReply::NetworkError)));
    connect(reply, SIGNAL(uploadProgress(qint64, qint64 )), this, SLOT(OnUploadProgress(qint64, qint64)));
}

void MainWindow::replyFinished(QNetworkReply *reply)
{
    QVariant status_code = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
    if(reply->error() == QNetworkReply::NoError)
    {
        qDebug()<< "-------------处理没有出错,开始同步个人中心输入总数---------";
        QByteArray bytes = reply->readAll();  //获取字节
        qDebug()<< "byte is "<<bytes;
        QString result(bytes);  //转化为字符串
        qDebug()<<result;

        QString user_total_filepath="/home/"+getCurrentLoginUser()+"/.config/fcitx/pinyin/inputHZcountTotal-"+email;

        qDebug()<< "文件路径为："<<user_total_filepath;

         QDir dir;
//        if(!dir.exists(user_total_dirpath))//判断需要创建的文件夹是否存在
//        {
//            dir.mkdir(user_total_dirpath); //创建文件夹
//        }


        qDebug()<<"本地文件记录的总数为："<<getFileContent(user_total_filepath).toInt();
        if (getFileContent(user_total_filepath).toInt()<result.toInt())
        {
            qDebug()<<"开始写文件";
            writefile(user_total_filepath,result);
            setCumVal(result.toInt());
        }
 //       writefile(user_total_filepath,result);

    }
    else
    {
        //处理错误
        qDebug()<<"Error" ;

    }
}

void MainWindow::slot_complete_sync(bool notError, int process) {
    qDebug()<<"Sync Complete!!"<<notError<<process;
    su->restartFcitx();//同步完成后重新启动fcitx输入法
    //ui->webView->page()->mainFrame()->evaluateJavaScript("showMask(false);$('#sync').val('立即同步');");//通知页面关闭遮罩层
    ui->webView->page()->mainFrame()->evaluateJavaScript("syncCumuWord()");//调用页面中同步累计输入字数方法
  //  ui->webView->page()->mainFrame()->evaluateJavaScript("syncTodayCumuWord()");//调用页面中同步累计输入字数方法
    uploadtodayinputnum();

}
void MainWindow::uploadtodayinputnum()
{
    QString mac=getLocalMac();
    QString mac_md5=md5_encode(mac);
    QDate datetime=QDate::currentDate();
    QString active_time=datetime.toString("yyyy-MM-dd");
    QString user_today_filepath="/home/"+getCurrentLoginUser()+"/.config/fcitx/pinyin/inputHZcoutToday-"+email;
    qDebug()<<"本地文件记录的总数为："<<getFileContent(user_today_filepath);

    QNetworkAccessManager *manager= new QNetworkAccessManager(this);
    QNetworkRequest *request = new QNetworkRequest();
    request->setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");

    QString test ="http://192.168.0.101:8080/im_service/user_linux/syncTodayWordCumu.action?";
  //  qDebug()<< "请求链接为："<<test;
    QString w_file = getFileContent(QString(WEB_CONFIG_FILE));
    URL = QString("http://%1").arg(w_file);
    QString url = QString("%1/im_service/user_linux/syncTodayWordCumu.action?").arg(URL);
    request->setUrl(QUrl(url));
    //request->setUrl(QUrl(test));
    QByteArray postData;
    postData.append("os=l");
    postData.append("&email=");
    postData.append(email);
    postData.append("&mac=");
    postData.append(mac);
    postData.append("&mac_md5=");
    postData.append(mac_md5);
    postData.append("&datetime=");
    postData.append(active_time);
    postData.append("&inputNum=");
    postData.append(getFileContent(user_today_filepath));


    QNetworkReply* reply = manager->post(*request, postData);
    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(upLoadError(QNetworkReply::NetworkError)));
    connect(reply, SIGNAL(uploadProgress(qint64, qint64 )), this, SLOT(OnUploadProgress(qint64, qint64)));
}
