#include "syncuserdb.h"

#include <QDebug>
#include <QProcess>
#include <QSettings>
#include <QCryptographicHash>

#include <unistd.h>
#include <utmp.h>
#include <QTextCodec>

#define URL_DOWNLOAD "%1/im_service/user_linux/downloadUserDB.action?os=l&email=%2"
#define DOWNLOAD_PATH "/usr/share/fcitx/personal-center/export.userdb.txt_server"

#define URL_UPLOAD "%1/im_service/userDBUploadServlet?os=l&email=%2&fileName=%3&ver=%4"

#define WEB_CONFIG_FILE "/usr/share/fcitx/personal-center/web_config"
#define USER_PROFILE "/usr/share/fcitx/personal-center/fcitx_user" //ini format file

#define USER_PHRASE_MB "/home/%1/.config/fcitx/pinyin/pyusrphrase.mb"
#define EXPORT_PATH "/usr/share/fcitx/personal-center/export.userdb.txt"
#define MERGE_PATH "/usr/share/fcitx/personal-center/export.userdb.txt_merge"
#define GBK_PY_ORG "/usr/share/fcitx/personal-center/gbkpy.org"
#define PY_PHRASE_MB "/usr/share/fcitx/personal-center/pyphrase.mb"
#define PY_USR_PHRASE_EXT_MB "/home/%1/.config/fcitx/pinyin/pyusrphrase_ext.mb"

SyncUserDB::SyncUserDB(QObject *parent) :
    QObject(parent)
{
    manager = new QNetworkAccessManager(this);
    connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(replyFinished(QNetworkReply*)));
    curProcess = PROCESS_STEP_INIT;
    reply = NULL;
    dwFile = NULL;
    upFileBuff = NULL;
}

SyncUserDB::~SyncUserDB()
{
    delete manager;
    if(reply != NULL) {
        delete reply;
        reply = NULL;
    }
    if(upFileBuff != NULL) {
        delete upFileBuff;
        upFileBuff = NULL;
    }
    if(dwFile != NULL) {
        delete dwFile;
        dwFile = NULL;
    }
}

bool SyncUserDB::init() {
    email = validateUserInfo();
    if(email == NULL) {
        return false;
    }
    QString w_file = getFileContent(QString(WEB_CONFIG_FILE));
    if(w_file != "") {
        baseURL = QString("http://%1").arg(w_file);
        return true;
    }
    ver = getFcitxVersion();
    return false;
}

void SyncUserDB::sync()
{
    this->clearTempFile();
    int rt = -1;
    rt = this->exportFile(EXPORT_PATH);
    if(rt == 0) {
        this->downlaod(QString(URL_DOWNLOAD).arg(baseURL).arg(email), DOWNLOAD_PATH);
    }
    else {
        qDebug()<<"Export Failure";
        emit completeSync(false, curProcess);
    }
    //-->replyFinished() curProcess==PROCESS_STEP_DOWNLOAD_COMPLETE
}

void SyncUserDB::replyFinished(QNetworkReply *reply)
{
    bool notError = true;
    //qDebug()<<"curProcess"<<curProcess;
    if (reply->error() == QNetworkReply::NoError) {
        if(curProcess == PROCESS_STEP_POST) {//postHttp
            qDebug()<<"PostHttp Complete!Reply: "<<reply->readAll();
            curProcess = PROCESS_STEP_POST_COMPLETE;
            emit completePost(notError, curProcess, QString(reply->readAll()));//
        }
        else if(curProcess == PROCESS_STEP_UPLOAD) {//upload
            qDebug() <<"Upload Complete!";
            curProcess = PROCESS_STEP_UPLOAD_COMPLETE;
            if(upFileBuff) {
                delete upFileBuff;
                upFileBuff = NULL;
            }
            //上传完成发送完成的信号
            emit completeSync(notError, curProcess);
        }
        else if(curProcess == PROCESS_STEP_DOWNLOAD) {//download
            qDebug() <<"Download Complete!";
            curProcess = PROCESS_STEP_DOWNLOAD_COMPLETE;
            dwFile->close();
            if(dwFile) {
                delete dwFile;
                dwFile = NULL;
            }
            int ret = -1;
            //merge and duplicate
            ret = this->merge(EXPORT_PATH, DOWNLOAD_PATH, MERGE_PATH);
            //import
            if(ret == 0) {
                //upload
                if(QFile::exists(MERGE_PATH)) {
                    ret = this->importFile(MERGE_PATH);
                    if(ret != 0) {
                        qDebug()<<"Import Failure";
                    }

                    this->upload(QString(URL_UPLOAD).arg(baseURL).arg(email).arg("pinyin.userdb.linux.txt").arg(this->getFcitxVersion()), MERGE_PATH);
                }
                else {
                    qDebug()<<MERGE_PATH<<"is not Exists";
                    notError = false;
                    emit completeSync(notError, curProcess);
                }
            }
            else if(ret == 1) {//服务端不存在可下载的文件
                qDebug()<<"Merge Failure - Server Download \"0\"";
                qDebug()<<"Upload Export File";
                if(QFile::exists(EXPORT_PATH)) {
                    this->upload(QString(URL_UPLOAD).arg(baseURL).arg(email).arg("pinyin.userdb.linux.txt").arg(this->getFcitxVersion()), EXPORT_PATH);
                }
                else {
                    qDebug()<<"Export File is not Exists";
                    notError = false;
                    emit completeSync(notError, curProcess);
                }
            }
            else {
                qDebug()<<"Merge Failure";
                notError = false;
                emit completeSync(notError, curProcess);
            }
        }
    }
    else {
        notError = false;
        emit completeSync(notError, curProcess);
        qDebug() <<"error! "<<curProcess;
    }
    reply->close();
    delete reply;
    //reply->disconnect(reply, SIGNAL(error(QNetworkReply::NetworkError)), 0, 0);
    //reply->disconnect(reply, SIGNAL(uploadProgress(qint64, qint64 )), 0, 0);
}

int SyncUserDB::downlaod(QString url, QString tarPath)
{
    qDebug()<<"Download Server Data."<<email <<"To" <<tarPath;
    curProcess = PROCESS_STEP_DOWNLOAD;
    if(dwFile) {
        delete dwFile;
    }
    dwFile = NULL;
    dwFile = new QFile(tarPath);
    if(!dwFile->open(QIODevice::WriteOnly)) {
        qDebug()<<"Can't open the file!"<<tarPath;
        return 1;
    }

    QUrl u(url);
    QNetworkRequest request(u);
    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/octet-stream");
    reply = manager->get(request);
    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(downloadError(QNetworkReply::NetworkError)));
    connect(reply, SIGNAL(readyRead()), this, SLOT(downlaodReadyRead()));
    connect(reply, SIGNAL(downloadProgress(qint64, qint64)), this, SLOT(downloadProgress(qint64, qint64)));
    return 0;
}

int SyncUserDB::upload(QString url, QString srcPath)
{
    qDebug()<<"Upload"<<srcPath;
    curProcess = PROCESS_STEP_UPLOAD;
    if(upFileBuff) {
        delete upFileBuff;
    }
    upFileBuff = NULL;
    //读取文件数据
    QFile file(srcPath);
    if(!file.open(QIODevice::ReadOnly)) {
        qDebug()<<"Can't open the file!"<<srcPath;
        return 1;
    }
    int len = file.size();
    QDataStream in(&file);
    upFileBuff = new char[len];
    in.readRawData(upFileBuff, len);
    file.close();

    //以二进制数据形式发送POST请求
    QUrl u(url);
    QNetworkRequest request(u);
    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/octet-stream");
    QByteArray arr = QByteArray(upFileBuff, len);
    reply = manager->post(request , arr);
    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(uploadError(QNetworkReply::NetworkError)));
    connect(reply, SIGNAL(uploadProgress(qint64, qint64 )), this, SLOT(uploadProgress(qint64, qint64)));
    return 0;
}

void SyncUserDB::postHttp(QString url, QByteArray &postData)
{
    curProcess = PROCESS_STEP_POST;
    QNetworkRequest *request = new QNetworkRequest();
    request->setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
    request->setUrl(QUrl(url));
    /*QNetworkReply* reply = */manager->post(*request, postData);
    //connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(postError(QNetworkReply::NetworkError)));
}

bool SyncUserDB::copyFileTo(QString src, QString to, bool coverFileIfExist) {
    if(src == to) {
        return true;
    }
    if(!QFile::exists(src)) {//source file
        qDebug()<<"Source file is not exist:"<<src;
        return false;
    }

    if(QFile::exists(to)) {
        if(coverFileIfExist) {
            QFile::remove(to);
        }
        else {
            qDebug()<<"To file is exist and not cover:"<<to;
            return false;//to file is exsits and not cover.
        }
    }
    return QFile::copy(src, to);
}

int SyncUserDB::importFile(QString srcFile)
{
    qDebug() <<"Import" <<srcFile;
    int ret = QProcess::execute("createPYMB", QStringList() <<GBK_PY_ORG<<srcFile);
    if(QFile::exists(PY_PHRASE_MB)) {
        //ret = QProcess::execute("mv", QStringList() <<PY_PHRASE_MB<<QString(PY_USR_PHRASE_EXT_MB).arg(this->getCurrentLoginUser()));
        //ret = QProcess::execute("mv", QStringList() <<"pyphrase.mb"<<QString(PY_USR_PHRASE_EXT_MB).arg(this->getCurrentLoginUser()));
        if(QFile::exists(PY_PHRASE_MB)) {
            bool rs = copyFileTo(PY_PHRASE_MB, QString(PY_USR_PHRASE_EXT_MB).arg(this->getCurrentLoginUser()), true);
            return !rs;
        }
        return 1;
    }
    else {
        qDebug() <<"Import Create mb File Failure!"<<srcFile;
    }
    return 1;
}

int SyncUserDB::exportFile(QString tar)
{
    qDebug()<<"Export User Phrase File:"<<tar;
    //mb2org -f pyusrphrase.mb > tar
    int rt = -1;
    rt = QProcess::execute("/bin/bash", QStringList()<<"-c"<<QString("mb2org -f %1 > %2").arg(QString(USER_PHRASE_MB).arg(this->getCurrentLoginUser())).arg(tar));
    /*QProcess proc;
    proc.start("mb2org", QStringList()<<"-f"<<QString(USER_PHRASE_MB).arg(this->getCurrentLoginUser()));
    proc.waitForFinished();
    rt = proc.exitCode();
    if(rt != 0) {//execute faiure
        return 1;
    }
    char data[1024];
    qint64 len;
    QFile tarFile(tar);
    if(!tarFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "Open failed:" << tarFile.fileName();
        return 2;
    }
    while(-1 != (len = proc.readLine(data, 1024))) {
        qDebug()<<data;
        QString s(data);
        tarFile.write(s.trimmed().toUtf8(), len);
        tarFile.write("\n");
    }
    proc.close();
    tarFile.close();

    rt = QFile::exists(tar) ? 0 : 3;*/
    return rt;
}

int SyncUserDB::merge(QString exportFile, QString serverFile, QString outputFile)
{
    if(!this->checkDBFile(serverFile)) {//check download file
        return 1;
    }
    qDebug()<<"Merge file "<<exportFile<<serverFile<<"to"<<outputFile;
    //cat file1 file2|sort|uniq > new.txt
    int ret = QProcess::execute("/bin/bash", QStringList()<<"-c"<<QString("cat %1 %2|sort|uniq > %3").arg(exportFile).arg(serverFile).arg(outputFile));
    return ret;

    /*QProcess proc;
    proc.start("/bin/bash", QStringList()<<"-c"<<QString("cat %1 %2|sort|uniq").arg(file1).arg(file2));
    proc.waitForFinished();
    //proc.waitForReadyRead();
    char data[1024];
    qint64 len;
    QFile tarFile(tar);
    if(!tarFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "Open failed:" << tarFile.fileName();
        return;
    }
    while(-1 != (len = proc.readLine(data, 1024))) {

        QString s(data);
        if(s.trimmed() != "") {
            qDebug() <<s.trimmed();

            tarFile.write(s.toUtf8());
            //tarFile.write("\n");
        }
    }
    proc.close();
    tarFile.close();*/
    return 0;
}

void SyncUserDB::clearTempFile() {
    qDebug()<<"Clear Temp File";
    //pybase.mb pyphrase.mb pyPhrase.ok pyERROR
    //QProcess::execute("rm", QStringList()<<"-rf"<<"./py*");//使用相对路径
    QProcess::execute("rm", QStringList()<<"-rf"<<"/usr/share/fcitx/personal-center/pybase.mb");
    QProcess::execute("rm", QStringList()<<"-rf"<<"/usr/share/fcitx/personal-center/pyphrase.mb");
    QProcess::execute("rm", QStringList()<<"-rf"<<"/usr/share/fcitx/personal-center/pyPhrase.ok");
    QProcess::execute("rm", QStringList()<<"-rf"<<"/usr/share/fcitx/personal-center/pyERROR");
    QProcess::execute("rm", QStringList()<<"-rf"<<"/usr/share/fcitx/personal-center/export.userdb.txt");
    QProcess::execute("rm", QStringList()<<"-rf"<<"/usr/share/fcitx/personal-center/export.userdb.txt_server");
    QProcess::execute("rm", QStringList()<<"-rf"<<"/usr/share/fcitx/personal-center/export.userdb.txt_merge");
}

void SyncUserDB::restartFcitx() {
    qDebug()<<"Restart Fcitx";
    //QProcess::execute("fcitx", QStringList()<<"-r");
    QProcess *p = new QProcess();
    p->start("fcitx", QStringList()<<"-r");
}

void SyncUserDB::uploadError(QNetworkReply::NetworkError errorCode)
{
    qDebug()<<"Upload ErrorCode:"<<errorCode;
}

void SyncUserDB::uploadProgress(qint64 bytesSent, qint64 bytesTotal)
{
    qDebug()<<"Upload Progress: "<<bytesSent<<"/"<<bytesTotal;
}

void SyncUserDB::downlaodReadyRead()
{
    if(reply->error() == QNetworkReply::NoError) {
        dwFile->write(this->reply->readAll());
    }
}

void SyncUserDB::downloadError(QNetworkReply::NetworkError errorCode)
{
    qDebug()<<"Download ErrorCode: "<<errorCode;
}

void SyncUserDB::downloadProgress(qint64 bytesReceived, qint64 bytesTotal)
{
    qDebug()<<"Download Progress: "<<bytesReceived<<"/"<<bytesTotal;
}

void SyncUserDB::postError(QNetworkReply::NetworkError errorCode)
{
    qDebug()<<"Post ErrorCode: "<<errorCode;
}

QString SyncUserDB::getFileContent(QString filePathName)
{
    QString fileContent = "";
    QFile tempfile(filePathName);
    if(!tempfile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Can't open the file!" << filePathName;
        return fileContent;
    }
    QTextStream txtInput(&tempfile);
    QTextCodec *code=QTextCodec::codecForName("UTF-8");
    txtInput.setCodec(code);
    QString lineStr;
    while(!txtInput.atEnd()) {
        lineStr = txtInput.readLine();
        //qDebug()<<lineStr;
        fileContent += lineStr;
    }
    tempfile.close();
    return fileContent;
}
/*
QString SyncUserDB::getCurrentUserAPPData() {
    //通过注册表查询用户是否自定义了用户目录路径，空串为使用了默认位置，既AppData目录
    QString appPath = getRegVal(USER_DIR, "RimeUserDir", "");
    //qDebug()<<appPath;
    if(appPath != NULL && appPath != "") {
        return appPath;
    }

    QStringList environmentList = QProcess::systemEnvironment();
    foreach (QString environment, environmentList) {
        if (environment.startsWith("APPDATA=", Qt::CaseInsensitive)) {
            appPath = environment.mid(QString("APPDATA=").length()) + "\\Rime";
            break;
        }
    }
    //qDebug()<<appPath;
    return appPath;
}*/

QString SyncUserDB::getCurrentLoginUser() {
    char* user = new char[512];
    strcpy(user, "");
    struct utmp record;
    FILE *fp;
    int strulen = sizeof(record);
    if((fp=fopen("/var/run/utmp", "r"))==0){
        perror("fopen");
        return NULL;
    }
    memset(&record, 0x00, strulen);
    time_t cur = time(NULL);
    while(fread(&record,strulen,1,fp)){
        if(record.ut_type == USER_PROCESS
                && strcmp(record.ut_host,":0") == 0
                && (strcmp(record.ut_line, "tty7") == 0 || strcmp(record.ut_line, "tty8") == 0)) {
            ////cout<<record.ut_user<<"----"<<record.ut_host<<"\t"<<record.ut_line<<endl;
            if(record.ut_tv.tv_sec < cur) {
                cur = record.ut_tv.tv_sec;
                strcpy(user, record.ut_user);
                break;
            }
        }
    }
    fclose(fp);
    ////cout<<"----"<<user<<"\t"<<cur<<endl;
    return QString(user);
}

QString SyncUserDB::getRegVal(QString path, QString key, QString default_val) {
    QSettings *st = new QSettings(path, QSettings::NativeFormat);
    QString rs = st->value(key, default_val).toString();
    delete st;
    return rs;
}

QString SyncUserDB::validateUserInfo()
{
    QString email64 = getRegVal(USER_PROFILE, "User/EM");
    QString md5 = getRegVal(USER_PROFILE, "User/MD");
    if(email64 == "" || md5 == "") return NULL;
    QString email = QString(QByteArray::fromBase64(email64.toLatin1()));
    QByteArray arr = QCryptographicHash::hash (email64.toLatin1(), QCryptographicHash::Md5);
    QString tar_md5(arr.toHex());
    QRegExp re("^[\\w-]+(\\.[\\w-]+)*@[\\w-]+(\\.[\\w-]+)+$");
    if(md5.toLower() == tar_md5.toLower() && re.exactMatch(email)) {
        return email;
    }
    return NULL;
}

QString SyncUserDB::getFcitxVersion() {
    QString v = "";
    QProcess process;
    process.start("dpkg", QStringList()<<"-s"<<"fcitx");
    process.waitForFinished();
    char data[1024];
    //Version: 1:4.2.8.3-3  ==> 4.2.8.3-3
    while(-1 != process.readLine(data, 1024)) {
        QString str = QString(data);
        if(str.startsWith("Version:")) {
            str = str.trimmed();
            v = str.mid(str.lastIndexOf(":") + 1);
            //qDebug()<<v.trimmed();
            break;
        }
    }
    process.close();
    v = QUrl::toPercentEncoding(v.toUtf8());//to encodeing
    //qDebug()<<v;
    return v;
}

bool SyncUserDB::checkDBFile(QString srcFile)
{
    if(!QFile::exists(srcFile)) {
        return false;
    }

    QFile file(srcFile);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug()<<"Can't open the file!"<<endl;
        return false;
    }
    QTextStream txtInput(&file);
    QString lineStr;
    if(!txtInput.atEnd()) {
        lineStr = txtInput.readLine();
        file.close();
        qDebug()<<"Check File Content:"<<lineStr;
        return !(/*lineStr == NULL || lineStr == "" || */lineStr == "\"0\"");
    }
    file.close();
    return false;
}
