/***************************************************************************
 *   Copyright (C) 2010~2010 by CSSlayer                                   *
 *   wengxt@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
 ***************************************************************************/

#include <dlfcn.h>
#include <libintl.h>
#include <pthread.h>
#include <stdio.h>
#include <string.h>


#include "fcitx/fcitx.h"
#include "module.h"
#include "addon.h"
#include "fcitx-config/xdg.h"
#include "fcitx-utils/log.h"
#include "instance.h"
#include "instance-internal.h"
#include "addon-internal.h"
#include "ime-internal.h"

void InitFcitxModules(UT_array* modules)
{
    utarray_init(modules, fcitx_ptr_icd);
}

FCITX_EXPORT_API
int FcitxStrReplace(char *p_result, char* p_source, char* p_seach, char *p_repstr)
{
    int c = 0;
    int repstr_leng = 0;
    int searchstr_leng = 0;
    char *p1;
    char *presult = p_result;
    char *psource = p_source;
    char *prep = p_repstr;
    char *pseach = p_seach;
    int nLen = 0;
    repstr_leng = strlen(prep);
    searchstr_leng = strlen(pseach);

    do {
        p1 = strstr(psource, p_seach);
        if (p1 == 0)
        {
            strcpy(presult, psource);
            return c;
        }
        c++;  //匹配子串计数加1;  
        //printf("结果:%s\r\n", p_result);  
        //printf("源字符:%s\r\n", p_source);  
        // 拷贝上一个替换点和下一个替换点中间的字符串  
        nLen = p1 - psource;
        memcpy(presult, psource, nLen);
        // 拷贝需要替换的字符串  
        memcpy(presult + nLen, p_repstr, repstr_leng);
        psource = p1 + searchstr_leng;
        presult = presult + nLen + repstr_leng;
    } while (p1);

    return c;
}

FCITX_EXPORT_API
void FcitxGetHotKey(char* result,char* configPath, char* hotkeyName)
{
        FILE *fp=fopen(configPath,"rw");
        if(fp == NULL)
        {
            return ;
        }

        char buf[1023];

        while(fgets(buf,1023,fp) != NULL)
        {
                int in=0;
                char *p[20];
                char *buffer = buf;
                if(strstr(buffer,hotkeyName)!=NULL)//RemindSwitchKey
                {

                        while((p[in]=strtok(buffer,"="))!=NULL)
                        {
                                in++;
                                buffer=NULL;
                        }
                        if(strcmp(p[1],"")!=0)
                        {
                                FcitxStrReplace(result,p[1],"_","+");

                        }
                }

        }
}


FCITX_EXPORT_API
void FcitxCmdShellResult(const char* command,char * result,int arrayLen)
{
        FILE *fpRead = NULL;
        char tmp[1024] = "";
        unsigned int count = 0;

        memset(result, 0, arrayLen);

        fpRead = popen(command, "r");
        if (NULL == fpRead)
                return ;
        while(NULL != fgets(tmp,1024-1,fpRead))
        {
                snprintf(result + count, arrayLen - count, "%s", tmp);
                count += strlen(tmp);
                if (count >= arrayLen)
                {
                        break;
                }
        }
        if (NULL != fpRead)
                pclose(fpRead);
        return ;
}




FCITX_EXPORT_API
void FcitxModuleLoad(FcitxInstance* instance)
{
    UT_array* addons = &instance->addons;
    FcitxAddon *addon;
    for (addon = (FcitxAddon *) utarray_front(addons);
            addon != NULL;
            addon = (FcitxAddon *) utarray_next(addons, addon)) {
        if (addon->bEnabled && addon->category == AC_MODULE) {
            char *modulePath;
            switch (addon->type) {
            case AT_SHAREDLIBRARY: {
                FILE *fp = FcitxXDGGetLibFile(addon->library, "r", &modulePath);
                void *handle;
                FcitxModule* module;
                void* moduleinstance = NULL;
                if (!fp)
                    break;
                fclose(fp);
                handle = dlopen(modulePath, RTLD_NOW | (addon->loadLocal ? RTLD_LOCAL : RTLD_GLOBAL));
                if (!handle) {
                    FcitxLog(ERROR, _("Module: open %s fail %s") , modulePath , dlerror());
                    break;
                }

                if (!FcitxCheckABIVersion(handle, addon->name)) {
                    FcitxLog(ERROR, "%s ABI Version Error", addon->name);
                    dlclose(handle);
                    break;
                }

                module = FcitxGetSymbol(handle, addon->name, "module");
                if (!module || !module->Create) {
                    FcitxLog(ERROR, _("Module: bad module"));
                    dlclose(handle);
                    break;
                }
                if ((moduleinstance = module->Create(instance)) == NULL) {
                    dlclose(handle);
                    break;
                }
                if (instance->loadingFatalError)
                    return;
                addon->module = module;
                addon->addonInstance = moduleinstance;
                if (module->ProcessEvent && module->SetFD)
                    utarray_push_back(&instance->eventmodules, &addon);
                utarray_push_back(&instance->modules, &addon);
            }
            break;
            default:
                break;
            }
            free(modulePath);
        }
    }
}

FCITX_EXPORT_API
FcitxModuleFunction
FcitxModuleFindFunction(FcitxAddon *addon, int func_id)
{
    if (!addon) {
        FcitxLog(DEBUG, "addon is not valid");
        return NULL;
    }

    /*
     * Input Methods support lazy load
     */
    if (addon->category == AC_INPUTMETHOD) {
        boolean flag = false;
        FcitxAddon **pimclass = NULL;
        for (pimclass = (FcitxAddon**)utarray_front(&addon->owner->imeclasses);
             pimclass;pimclass = (FcitxAddon**)utarray_next(
                 &addon->owner->imeclasses, pimclass)) {
            if (*pimclass == addon) {
                flag = true;
                break;
            }
        }
        if (!flag && !addon->addonInstance) {
            FcitxInstanceLoadIM(addon->owner, addon);
            FcitxInstanceUpdateIMList(addon->owner);
        }
    }
    FcitxModuleFunction *func_p = fcitx_array_eltptr(&addon->functionList,
                                                     func_id);
    if (func_p)
        return *func_p;
    return NULL;
}

FCITX_EXPORT_API void*
FcitxModuleInvokeOnAddon(FcitxAddon *addon, FcitxModuleFunction func,
                         FcitxModuleFunctionArg *args)
{
    if (!func)
        return NULL;
    return func(addon->addonInstance, *args);
}

static void*
_FcitxModuleInvokeFunction(FcitxAddon* addon, int functionId,
                           FcitxModuleFunctionArg args)
{
    FcitxModuleFunction func = FcitxModuleFindFunction(addon, functionId);
    if (!func) {
        FcitxLog(DEBUG, "addon %s doesn't have function with id %d",
                 addon->name, functionId);
        return NULL;
    }
    return func(addon->addonInstance, args);
}

FCITX_EXPORT_API void*
FcitxModuleInvokeFunction(FcitxAddon* addon, int functionId,
                          FcitxModuleFunctionArg args)
{
    return _FcitxModuleInvokeFunction(addon, functionId, args);
}

FCITX_EXPORT_API
void* FcitxModuleInvokeFunctionByName(FcitxInstance* instance, const char* name, int functionId, FcitxModuleFunctionArg args)
{
    FcitxAddon* module = FcitxAddonsGetAddonByName(&instance->addons, name);

    if (module == NULL) {
        return NULL;
    } else {
        return _FcitxModuleInvokeFunction(module, functionId, args);
    }
}

FCITX_EXPORT_API void
FcitxModuleAddFunction(FcitxAddon *addon, FcitxModuleFunction func)
{
    void *temp = (void*)func;
    utarray_push_back(&addon->functionList, &temp);
}

// kate: indent-mode cstyle; space-indent on; indent-width 0;
