﻿#include "dialog.h"
#include "ui_dialog.h"
#include <iostream>
//#include <QPixmap>
//#include <QApplication>
#include <QScreen>
#include <QMessageBox>
#include <QRegExp>

Dialog::Dialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Dialog)
{
    ui->setupUi(this);
    init_description = tr("In order to protect your privacy, make sure your feedback does not contain private information.Please fill in the blanks as far as possible. For example, if you want to provide feedback about the collapse of the input method, please tell us which application you are using. If possible, please provide us with the question of whether the application has just started, or the implementation of some specific operations, it is best to provide what steps to reproduce the problem.");
    setWindowFlags(Qt::CustomizeWindowHint | Qt::WindowCloseButtonHint);
    manager = new QNetworkAccessManager(this);
    this->setFixedSize(550,582);

    //connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(addFeedback(QNetworkReply*)));


    QString title = ui->f_Title->text();
    int exitCode = -1;
    QProcess process;
    #ifdef Q_OS_WIN
        qDebug() << "windows";
        process.start("ver");
        process.waitForFinished();
        exitCode = process.exitCode();

        //exitCode = QProcess::execute("systeminfo");
        //exitCode = process.execute("ver");
    #else
        qDebug() << "linux";
        /*
        process.start("lsb_release -a | grep Description | awk '{print $2}'");
        process.waitForFinished();
        exitCode = process.exitCode();

        exitCode = QProcess::execute("lsb_release -a | grep Description | awk '{print $2}'");
        exitCode = process.execute("lsb_release -a | grep Description | awk '{print $2}'");
        */
        QProcess process1;
        process1.setStandardOutputProcess(&process);

        process1.start("lsb_release -a");
        process1.waitForFinished();

        process.start("grep Description");
        process.waitForFinished();

        exitCode = process.exitCode();
    #endif

    qDebug() << exitCode;
    if(exitCode == 0)
    {
       /* QString OSVersion = QString(process.readAllStandardOutput());
        int index = OSVersion.indexOf(':');
        OSVersion = OSVersion.mid(index + 1).trimmed();
        index = OSVersion.indexOf(' ');
        title = OSVersion.mid(0, index) + title;*/
        title= tr("Fcitx %1").arg(title);
        ui->f_Title->setText(title);
    }
    //ui->progressBar->hide();
}

Dialog::~Dialog()
{
    delete ui;
}

void Dialog::on_pushButton_clicked()
{

    QRegExp regExp("^([a-zA-Z0-9_-])+@([a-zA-Z0-9_-])+((\.[a-zA-Z0-9_-]{2,3}){1,2})$");
    // QRegExp regExp_en("[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z]+)+");
    if(ui->f_Mail->text()!="")
    {
        qDebug()<<ui->f_Mail->text().length();
        if( ui->f_Mail->text().indexOf(regExp)==0 && ui->f_Mail->text().length()<= 64)
            WriteSettings(ui->f_Mail->text());
        else
        {
            QMessageBox::critical(0, tr("The mail is invalid"), tr("Please enter the correct mail."), tr("OK"));
            ui->textEdit->setDisabled(false);
            ui->f_Mail->setDisabled(false);
            //ui->checkBox->setDisabled(false);
            return;
        }
    }

    //ui->progressBar->setMaximum(100);
    //ui->progressBar->setValue(30);
//    ui->progressBar->show();

    //ui->textEdit->setReadOnly(true);
    ui->textEdit->setDisabled(true);
    ui->f_Mail->setDisabled(true);


    QString description = ui->textEdit->document()->toPlainText().trimmed();
    if((description == init_description) || (description == ""))
    {
        QMessageBox::critical(0, QString::fromUtf8("问题描述为空"), QString::fromUtf8("请详细描述您遇到的问题，以便得到快速反馈。"), QString::fromUtf8("确定"));
        ui->textEdit->setDisabled(false);
        ui->f_Mail->setDisabled(false);
        return;
    }


    QString email = ui->f_Mail->text();

    //提交反馈
    QNetworkRequest req;
    //req.setHeader(QNetworkRequest::ContentTypeHeader, "application/octet-stream");
    req.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");

    QUrl url(QString::fromUtf8("%1").arg(base_url));
    req.setUrl(url);
    //QByteArray append((const char*)("description=" + description + "&email=" + email).utf16());
    //QByteArray append(("description=" + description + "&email=" + email + "&screenshot=" + screenshot).toLatin1());
    QByteArray append("description=" + QUrl::toPercentEncoding(description));
    append.append("&email=" + QUrl::toPercentEncoding(email.toUtf8()));
    //append.append("&screenshot=" + QUrl::toPercentEncoding(screenshot));
    append.append("&screenshot=" + QUrl::toPercentEncoding(screenshot));
    qDebug() << append.size();
    //manager->post(QNetworkRequest(url), append);
    reply = manager->post(req, append);
    connect(reply, SIGNAL(readyRead()), this, SLOT(httpReadyRead()));
    connect(reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(updateDataReadProgress(qint64,qint64)));
    connect(reply, SIGNAL(finished()), this, SLOT(httpFinished()));
}

void Dialog::addFeedback(QNetworkReply* reply)
{
    if(reply->error() == QNetworkReply::NoError)
    {
        QByteArray bytes = reply->readAll();
        QString result = QString::fromUtf8(bytes);

        //std::cout << "提交反馈" << result.toStdString() << "!" << std::endl;
        //qDebug() << "提交反馈" << result << "!";
        qDebug() << "提交反馈" + result + "!";
        qDebug() << "=============================";
    }
    else
    {

    }
}

void Dialog::httpReadyRead()
{
    addFeedback(reply);
}

void Dialog::updateDataReadProgress(qint64 bytesRead, qint64 totalBytes)
{
    //ui->progressBar->setMaximum(totalBytes);
    //ui->progressBar->setValue(bytesRead);
}

void Dialog::httpFinished()
{
    QVariant statusCodeV = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
    qDebug() << statusCodeV;
    QVariant redirectionTargetUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
    qDebug() << redirectionTargetUrl;
    qDebug() << reply->request().url();

    //ui->progressBar->hide();
    reply->deleteLater();
    reply =0;
    //删除截图文件
    remove(qPrintable(fileName));

    if(statusCodeV.isValid() && statusCodeV.toInt() < 400)
    {
        /*
        int ok = QMessageBox::warning(this, QString("提交反馈"), QString("你确定提交此次反馈吗？"), QString("确定"), QString("取消"));
        if(ok == 0)
        {
        }
        */
        QMessageBox::information(this, QString::fromUtf8("提交反馈"), QString::fromUtf8("你的反馈已经成功提交！"), QString::fromUtf8("确定"));
        //关闭对话框
        accept();
    }
    else
    {
        QMessageBox::critical(0, QString::fromUtf8("提交失败"), QString::fromUtf8("请求服务无响应，请稍后再试！"), QString::fromUtf8("确定"));
        ui->textEdit->setDisabled(false);
        ui->f_Mail->setDisabled(false);
    }
}

void Dialog::on_textEdit_cursorPositionChanged()
{
    QString description = ui->textEdit->document()->toPlainText();

    if(description == init_description)
    {
        ui->textEdit->document()->clear();
    }
}

void Dialog::WriteSettings(QString mail)

{
    QSettings settings(this->userMail, QSettings::IniFormat); // 当前目录的INI文件

    //    settings.beginGroup("MailName");

    settings.setValue("mailName", mail);

    settings.endGroup();

}

void Dialog::on_textEdit_textChanged()
{
    QString textContent = ui->textEdit->toPlainText();
    int length = textContent.count();
    int maxLen = 256;
    int maxLength = maxLen; // 最大字符数
    if(length > maxLength)
    {
        QMessageBox::warning(0, tr("info"), tr("Enter more words than limit!"), tr("OK"));
        //无法继续输入
        int position = ui->textEdit->textCursor().position();
        QTextCursor textCursor = ui->textEdit->textCursor();
        textContent.remove(position - (length - maxLength), length - maxLength);
        ui->textEdit->setText(textContent);
        textCursor.setPosition(position - (length - maxLength));
        ui->textEdit->setTextCursor(textCursor);
    }
}
