/***************************************************************************
 *   Copyright (C) 2002~2005 by Yuking                                     *
 *   yuking_net@sohu.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.             
 *   Copyright (C) 2016-2018 CPU and Fundamental Software Research Center, Chinese Academy of Sciences.
 ***************************************************************************/

#include <X11/Xutil.h>
#include <X11/extensions/Xrender.h>
#include <X11/Xatom.h>
#include <unistd.h>
#include <cairo.h>
#include <limits.h>
#include <libintl.h>
#include <errno.h>

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>
#include <math.h>
#include <pwd.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "fcitx/fcitx.h"
#include "fcitx/ui.h"
#include "fcitx/module.h"

#include "classicui.h"
#include "classicuiinterface.h"
#include "fcitx-config/xdg.h"
#include "fcitx-utils/log.h"
#include "fcitx/instance.h"
#include "fcitx/instance-internal.h"
#include "fcitx/frontend.h"
#include "InputWindow.h"
#include "MainWindow.h"
#include "TrayWindow.h"
#include "MenuWindow.h"
#include "fcitx/hook.h"
#include "fcitx-utils/utils.h"
#include "module/notificationitem/fcitx-notificationitem.h"
#include <stdio.h>    // for printf()  
#include <unistd.h>   // for pause()  
#include <signal.h>   // for signal()  
#include <string.h>   // for memset()  
#include <sys/time.h>
struct _FcitxSkin;
static boolean MainMenuAction(FcitxUIMenu* menu, int index);
static boolean IMMenuAction(FcitxUIMenu *menu, int index);
static void UpdateMainMenu(FcitxUIMenu* menu);
static void UpdateleftMainMenu(FcitxUIMenu* menu);
static void* ClassicUICreate(FcitxInstance* instance);
static void ClassicUICloseInputWindow(void* arg);
static void ClassicUIShowInputWindow(void* arg);
static void ClassicUIMoveInputWindow(void* arg);
static void ClassicUIRegisterMenu(void *arg, FcitxUIMenu* menu);
static void ClassicUIUnRegisterMenu(void *arg, FcitxUIMenu* menu);
static void ClassicUIUpdateStatus(void *arg, FcitxUIStatus* status);
static void ClassicUIRegisterStatus(void *arg, FcitxUIStatus* status);
static void ClassicUIUpdateComplexStatus(void *arg, FcitxUIComplexStatus* status);
static void ClassicUIRegisterComplexStatus(void *arg, FcitxUIComplexStatus* status);
static void ClassicUIOnInputFocus(void *arg);
static void ClassicUIOnInputUnFocus(void *arg);
static void ClassicUIOnTriggerOn(void *arg);
static void ClassicUIOnTriggerOff(void *arg);
static void ClassicUIInputReset(void *arg);
static void ReloadConfigClassicUI(void *arg);
static void ClassicUISuspend(void *arg);
static void ClassicUIResume(void *arg);
static void ClassicUIDelayedInitTray(void* arg);
static void ClassicUIDelayedShowTray(void* arg);
static void ClassicUINotificationItemAvailable(void* arg, boolean avaiable);

static FcitxConfigFileDesc* GetClassicUIDesc();
static void ClassicUIMainWindowSizeHint(void *arg, int* x, int* y,
                                        int* w, int* h);

/////////Update
char *fcitx_install_package[5] = {"fcitx","fcitx-config","cloudpinyin","charpainting-data","fcitx-cell-list"};

DECLARE_ADDFUNCTIONS(ClassicUI)

FCITX_DEFINE_PLUGIN(fcitx_classic_ui, ui, FcitxUI) = {
    ClassicUICreate,
    ClassicUICloseInputWindow,
    ClassicUIShowInputWindow,
    ClassicUIMoveInputWindow,
    ClassicUIUpdateStatus,
    ClassicUIRegisterStatus,
    ClassicUIRegisterMenu,
    ClassicUIOnInputFocus,
    ClassicUIOnInputUnFocus,
    ClassicUIOnTriggerOn,
    ClassicUIOnTriggerOff,
    NULL,
    ClassicUIMainWindowSizeHint,
    ReloadConfigClassicUI,
    ClassicUISuspend,
    ClassicUIResume,
    NULL,
    ClassicUIRegisterComplexStatus,
    ClassicUIUpdateComplexStatus,
    ClassicUIUnRegisterMenu,
};

/********/
#if 0
//Temporarily remove the update function
void TimetoUpdate()
{

	printf("TimetoUpdate\n");
        popen("/usr/share/fcitx/update/CheckUpgradeExit.sh","r");
	//
}

void * dingshiThread()
{ printf("$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n");
    int res = 0;  
  
  // Register printMsg to SIGALRM  
  
  signal(SIGALRM, TimetoUpdate);  
  struct itimerval tick;  
  
  // Initialize struct  
  
  memset(&tick, 0, sizeof(tick));  
  
  // Timeout to run function first time  
  
  tick.it_value.tv_sec = 1;  // sec  
  
  tick.it_value.tv_usec = 0; // micro sec.  
  
  // Interval time to run function  
  
  tick.it_interval.tv_sec = 14400;  
  
  tick.it_interval.tv_usec = 0;  
  
  // Set timer, ITIMER_REAL : real-time to decrease timer,  
  
  //                          send SIGALRM when timeout  
  
  res = setitimer(ITIMER_REAL, &tick, NULL);  
  
  if (res) {  
  
    printf("Set timer failed!!\n");  }  
  
  // Always sleep to catch SIGALRM signal  
  
   
}

void set_dingshi_thread()
{
	int err;
	pthread_t dingshi_thread;
	if(( err=pthread_create(&dingshi_thread,NULL,dingshiThread,NULL))!=0)
	{ 
		 return 1;
	}
	printf("dingshixiancheng---------------------------------------------\n");

}
#endif
int CheckUserConffile(char *m_filepath)
{
	FILE * fp;
	char buf[1024];
	int flag =0;
	int flag_x =0;
	int flag_y =0;
//	fp=fopen("/home/nfs/.config/fcitx/conf/fcitx-classic-ui.config","r");
	fp =fopen(m_filepath,"r");
	if(fp==NULL)  
    	{
        	perror("open file");
		return 1;    
    	}
	while(fgets(buf,sizeof(buf),fp)!=NULL)  
    	{    
        	printf("%s", buf);
		if (strstr(buf,"\#MainWindowOffsetX")!=NULL)
		{
			printf("X user not set\n");
			flag_x = 1;
		}
		else if (strstr(buf,"\#MainWindowOffsetY")!=NULL)
		{
			printf("Y user not set \n");
			flag_y = 1;
		}

		if ((flag_x == 1)&& (flag_y == 1))
		{
			printf("User not set X and Y \n");
			flag =1;
			break;	
		}

    	}
	fclose(fp);
	return flag;  	  

}

/****update****/

void cmd_shell_result(const char* command,char * result,int length)
{
    FILE *fpRead;
    fpRead = popen(command, "r");
    char buf[1024];
    memset(buf,'\0',sizeof(buf));
    while(fgets(buf,1024-1,fpRead)!=NULL)
    {
        strncpy(result,buf,length);
        result = buf;
    }
    if(fpRead!=NULL)
        pclose(fpRead);
    return ;
}
int Check_Software_Version_IsorNotNew(char *m_softwarename)
{
    char apt_fcitx_version[512];
    char sys_fcitx_version[512];
    memset(apt_fcitx_version,0,512);
    memset(sys_fcitx_version,0,512);

    char software_realname[1024];


    if (strcmp(m_softwarename,"fcitx")==0)
    {
	cmd_shell_result("apt-cache show fcitx | grep Version | head -n 1 | cut -d ':' -f3",apt_fcitx_version,sizeof(apt_fcitx_version));
    	cmd_shell_result("dpkg -s fcitx | grep Version | cut -d ':' -f3",sys_fcitx_version,sizeof(sys_fcitx_version));
	
    }
    else
    {
	char get_sys_version_commamd[1024];
	char get_apt_version_command[1024];
	
	memset(get_sys_version_commamd,0,1024);
	memset(get_apt_version_command,0,1024);

	snprintf(get_sys_version_commamd,sizeof(get_sys_version_commamd),"softwarename=`dpkg -l | grep %s |awk '{print $2}'| head -n 1`;dpkg -s $softwarename | grep Version | cut -d ' ' -f2",m_softwarename);
	snprintf(get_apt_version_command,sizeof(get_apt_version_command),"softwarename=`dpkg -l | grep %s |awk '{print $2}'| head -n 1`;apt-cache show $softwarename | grep Version | head -n 1 | cut -d ' ' -f2",m_softwarename);
	
	cmd_shell_result(get_apt_version_command,apt_fcitx_version,sizeof(apt_fcitx_version));
    	cmd_shell_result(get_sys_version_commamd,sys_fcitx_version,sizeof(sys_fcitx_version));

	apt_fcitx_version[strlen(apt_fcitx_version)-1]=0;
    	sys_fcitx_version[strlen(sys_fcitx_version)-1]=0;
	
    }
   
    if (strcmp(apt_fcitx_version,sys_fcitx_version)==0)
    {
	printf("system is new version\n");
	return 0;
    }
    else
    {
	printf("system is not new version\n");
        return 1;
    }

}

void Update_Fcitx()
{
/*	char install_list[1024]="";
	int fcitx_install_flag = 0;
	int fcitx_update_flag = 0;
	for (int i =0; i<5;i++)
	{
		printf("value is %s\n",fcitx_install_package[i]);
		if (Check_Software_Version_IsorNotNew(fcitx_install_package[i])==1)
		{
			if (strcmp(fcitx_install_package[i],"fcitx") == 0)
			{
				fcitx_install_flag = 1;
			}
			strcat(install_list,fcitx_install_package[i]);
			strcat(install_list,",");
			fcitx_update_flag = 1;
			
		}

	}	
	printf("list is %s    flag is %d\n",install_list,fcitx_install_flag);
	char result_run_update[512];
	memset(result_run_update,0,512);
	cmd_shell_result("ps -aux | grep fcitx-upgrade | grep -v grep | wc -l",result_run_update,sizeof(result_run_update));
	result_run_update[strlen(result_run_update)-1]=0;
	
	if (strcmp(result_run_update,"0")==0)
	{
		if (fcitx_update_flag == 0)
		{
			popen("/usr/share/fcitx/update/fcitx-upgrade noupdate","r");
		}
		else
		{
			char open_update_command[1024];
			memset(open_update_command,0,sizeof(open_update_command));
			snprintf(open_update_command,sizeof(open_update_command),"/usr/share/fcitx/update/fcitx-upgrade %s",install_list);
			printf("open update command is %s\n",open_update_command);
			popen(open_update_command,"r");
		
		}
	}*/
        char result_run_update[512];
	memset(result_run_update,0,512);
	cmd_shell_result("ps -aux | grep fcitx-upgrade | grep -v grep | wc -l",result_run_update,sizeof(result_run_update));
	result_run_update[strlen(result_run_update)-1]=0;
	
	if (strcmp(result_run_update,"0")==0)
	{
		popen("/usr/share/fcitx/update/fcitx-upgrade","r");
	}
	
}

/*************/
/////////ADD BY YL///////////////////
void UpdateCharMenu(FcitxUIMenu* menu)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) menu->priv;
    FcitxMenuClear(menu);

    FcitxMenuAddMenuItem(menu, _("CharExpression"), MENUTYPE_SIMPLE, NULL);
    FcitxMenuAddMenuItem(menu, _("CharPainting"), MENUTYPE_SIMPLE, NULL);
    FcitxMenuAddMenuItem(menu, _("Date"), MENUTYPE_SIMPLE, NULL);
}
boolean CharMenuAction(FcitxUIMenu* menu, int index)
{
    char charMenuCommand[1024];
    char args[1024];
    char buf[1024];
    FILE *stream;
    if(index == 0)
    {
	strncpy(charMenuCommand,"/usr/share/fcitx/data/charpainting/XPainting charexpression",sizeof(charMenuCommand));
	strncpy(args,"charexpression",sizeof(args));
    }
    else if(index == 1)
    {
	strncpy(charMenuCommand,"/usr/share/fcitx/data/charpainting/XPainting asciiart",sizeof(charMenuCommand));
	        strncpy(args,"asciiart",sizeof(args));

    }
    else if(index == 2)
    {
	strncpy(charMenuCommand,"/usr/share/fcitx/data/charpainting/XPainting dateandtime",sizeof(charMenuCommand));
	        strncpy(args,"dateandtime",sizeof(args));

    }
    char cmd[1024];
    strncpy(cmd,"/usr/share/fcitx/data/charpainting/Check_Run_Only_Once.sh  ",sizeof(cmd));
    strcat(cmd,args);
    stream=popen(cmd,"r");
    fread( buf, sizeof(char), sizeof(buf),  stream);
    pclose(stream);
    printf("buf  is %d\n",atoi(buf));
    if(atoi(buf)==2)
    {
	printf("not exit start\n");
    	popen(charMenuCommand, "r");
    }
    return true;
}
void InitCharMenu(FcitxClassicUI* classicui)
{
    FcitxMenuInit(&classicui->charMenu);
    classicui->charMenu.candStatusBind = NULL;
    classicui->charMenu.name =  strdup(_("Expression&Symbol"));

    classicui->charMenu.UpdateMenu = UpdateCharMenu;
    classicui->charMenu.MenuAction = CharMenuAction;
    classicui->charMenu.priv = classicui;
    classicui->charMenu.isSubMenu = false;
}
////////////END BY YL//////////////////

void* ClassicUICreate(FcitxInstance* instance)
{
    FcitxAddon *classicuiaddon = Fcitx_ClassicUI_GetAddon(instance);
    FcitxClassicUI *classicui = fcitx_utils_new(FcitxClassicUI);
    classicui->owner = instance;
    if (!LoadClassicUIConfig(classicui)) {
        free(classicui);
        return NULL;
    }
    if (GetSkinDesc() == NULL) {
        free(classicui);
        return NULL;
    }
    classicui->dpy = FcitxX11GetDisplay(instance);
    if (classicui->dpy == NULL) {
        free(classicui);
        return NULL;
    }

    FcitxX11GetDPI(instance, &classicui->dpi, NULL);
    if (classicui->dpi <= 0)
        classicui->dpi = 96;

    if (LoadSkinConfig(&classicui->skin, &classicui->skinType)) {
        free(classicui);
        return NULL;
    }

    classicui->isfallback = FcitxUIIsFallback(instance, classicuiaddon);

    classicui->iScreen = DefaultScreen(classicui->dpy);

    classicui->protocolAtom = XInternAtom(classicui->dpy, "WM_PROTOCOLS", False);
    classicui->killAtom = XInternAtom(classicui->dpy, "WM_DELETE_WINDOW", False);


    InitSkinMenu(classicui);
    FcitxUIRegisterMenu(instance, &classicui->skinMenu);
//////////////
    //InitCharMenu(classicui);
   // FcitxUIRegisterMenu(instance, &classicui->charMenu);
///////////////

    /* Main Menu Initial */
    FcitxMenuInit(&classicui->mainMenu);
    classicui->mainMenu.UpdateMenu = UpdateMainMenu;
    classicui->mainMenu.MenuAction = MainMenuAction;
    classicui->mainMenu.priv = classicui;
    classicui->mainMenu.mark = -1;

    FcitxMenuInit(&classicui->leftmainMenu);
    classicui->leftmainMenu.UpdateMenu = UpdateleftMainMenu;
    classicui->leftmainMenu.MenuAction = IMMenuAction;
    classicui->leftmainMenu.priv = classicui;
    classicui->leftmainMenu.mark = -1;


    classicui->inputWindow = InputWindowCreate(classicui);
    classicui->mainWindow = MainWindowCreate(classicui);
    classicui->trayWindow = TrayWindowCreate(classicui);
    classicui->mainMenuWindow = MainMenuWindowCreate(classicui);
    classicui->leftmainMenuWindow = leftMainMenuWindowCreate(classicui);

    FcitxIMEventHook resethk;
    resethk.arg = classicui;
    resethk.func = ClassicUIInputReset;
    FcitxInstanceRegisterResetInputHook(instance, resethk);

    DisplaySkin(classicui, classicui->skinType);

    FcitxClassicUIAddFunctions(instance);

    classicui->waitDelayed = FcitxInstanceAddTimeout(instance, 0, ClassicUIDelayedInitTray, classicui);

    return classicui;
}

void ClassicUIDelayedInitTray(void* arg) {
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    // FcitxLog(INFO, "yeah we delayed!");
    if (!classicui->bUseTrayIcon)
        return;
    /*
     * if this return false, something wrong happened and callback
     * will never be called, show tray directly
     */
    if (FcitxNotificationItemEnable(classicui->owner, ClassicUINotificationItemAvailable, classicui)) {
        if (!classicui->trayTimeout)
            classicui->trayTimeout = FcitxInstanceAddTimeout(classicui->owner, 100, ClassicUIDelayedShowTray, classicui);
    } else {
        TrayWindowRelease(classicui->trayWindow);
        TrayWindowInit(classicui->trayWindow);
    }
}

void ClassicUIDelayedShowTray(void* arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    classicui->trayTimeout = 0;
    if (!classicui->bUseTrayIcon)
        return;

    if (!classicui->trayWindow->bTrayMapped) {
        TrayWindowRelease(classicui->trayWindow);
        TrayWindowInit(classicui->trayWindow);
    }
}

void ClassicUISetWindowProperty(FcitxClassicUI* classicui, Window window, FcitxXWindowType type, char *windowTitle)
{
    FcitxX11SetWindowProp(classicui->owner, &window, &type, windowTitle);
}

static void ClassicUIInputReset(void *arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    if (classicui->isSuspend)
        return;
    MainWindowShow(classicui->mainWindow);
    TrayWindowDraw(classicui->trayWindow);
    classicui->inputWindow->highlight = 0;
}

static void ClassicUICloseInputWindow(void *arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    InputWindowClose(classicui->inputWindow);
}

static void ClassicUIShowInputWindow(void *arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    InputWindowShow(classicui->inputWindow);
}

static void ClassicUIMoveInputWindow(void *arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    classicui->inputWindow->parent.MoveWindow(&classicui->inputWindow->parent);
}

static void ClassicUIUpdateStatus(void *arg, FcitxUIStatus* status)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    MainWindowShow(classicui->mainWindow);
}

void ClassicUIUpdateComplexStatus(void* arg, FcitxUIComplexStatus* status)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    MainWindowShow(classicui->mainWindow);
}

void ClassicUIRegisterComplexStatus(void* arg, FcitxUIComplexStatus* status)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    status->uipriv[classicui->isfallback] = fcitx_utils_malloc0(sizeof(FcitxClassicUIStatus));
}


static void ClassicUIRegisterMenu(void *arg, FcitxUIMenu* menu)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    XlibMenu* xlibMenu = XlibMenuCreate(classicui);
    menu->uipriv[classicui->isfallback] = xlibMenu;
    xlibMenu->menushell = menu;
}

static void ClassicUIUnRegisterMenu(void *arg, FcitxUIMenu* menu)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    XlibMenuDestroy((XlibMenu*) menu->uipriv[classicui->isfallback]);
}

static void ClassicUIRegisterStatus(void *arg, FcitxUIStatus* status)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    FcitxSkin* sc = &classicui->skin;
    status->uipriv[classicui->isfallback] = fcitx_utils_new(FcitxClassicUIStatus);
    char *name;

    fcitx_utils_alloc_cat_str(name, status->name, "_active.png");
    LoadImage(sc, name, false);
    free(name);

    fcitx_utils_alloc_cat_str(name, status->name, "_inactive.png");
    LoadImage(sc, name, false);
    free(name);
}

static void ClassicUIOnInputFocus(void *arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    if (classicui->isSuspend)
        return;
    MainWindowShow(classicui->mainWindow);
    TrayWindowDraw(classicui->trayWindow);
}

static void ClassicUIOnInputUnFocus(void *arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    if (classicui->isSuspend)
        return;
    MainWindowShow(classicui->mainWindow);
    TrayWindowDraw(classicui->trayWindow);
}

void ClassicUISuspend(void* arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    classicui->isSuspend = true;
    classicui->notificationItemAvailable = false;
    InputWindowClose(classicui->inputWindow);
    MainWindowClose(classicui->mainWindow);
    TrayWindowRelease(classicui->trayWindow);
    /* always call this function will not do anything harm */
    FcitxNotificationItemDisable(classicui->owner);
}

void ClassicUIResume(void* arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    classicui->isSuspend = false;
    ClassicUIDelayedInitTray(classicui);
}

void ClassicUINotificationItemAvailable(void* arg, boolean avaiable) {
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    /* ClassicUISuspend has already done all clean up */
    if (classicui->isSuspend)
        return;
    classicui->notificationItemAvailable = avaiable;
    if (!avaiable) {
        TrayWindowRelease(classicui->trayWindow);
        TrayWindowInit(classicui->trayWindow);
    } else {
        if (classicui->trayTimeout) {
            FcitxInstanceRemoveTimeoutById(classicui->owner, classicui->trayTimeout);
            classicui->trayTimeout = 0;
        }
        TrayWindowRelease(classicui->trayWindow);
    }
}

void ActivateWindow(Display *dpy, int iScreen, Window window)
{
    XEvent ev;

    memset(&ev, 0, sizeof(ev));

    static Atom _NET_ACTIVE_WINDOW;
    if (_NET_ACTIVE_WINDOW == None)
        _NET_ACTIVE_WINDOW = XInternAtom(dpy, "_NET_ACTIVE_WINDOW", False);

    ev.xclient.type = ClientMessage;
    ev.xclient.window = window;
    ev.xclient.message_type = _NET_ACTIVE_WINDOW;
    ev.xclient.format = 32;
    ev.xclient.data.l[0] = 1;
    ev.xclient.data.l[1] = CurrentTime;
    ev.xclient.data.l[2] = 0;

    XSendEvent(dpy, RootWindow(dpy, iScreen), False, SubstructureNotifyMask, &ev);
    XSync(dpy, False);
}

FcitxRect GetScreenGeometry(FcitxClassicUI* classicui, int x, int y)
{
    FcitxRect result = { 0, 0 , 0 , 0 };
    FcitxX11GetScreenGeometry(classicui->owner, &x, &y, &result);
    return result;
}

CONFIG_DESC_DEFINE(GetClassicUIDesc, "fcitx-classic-ui.desc")

boolean LoadClassicUIConfig(FcitxClassicUI* classicui)
{
    struct passwd *pwd;
    pwd = getpwuid(getuid());
    printf("当前登录的用户名为:%s\n",pwd->pw_name);
    char usr_conffile[1024];
    memset(usr_conffile,0,1024);
    snprintf(usr_conffile,sizeof(usr_conffile),"/home/%s/.config/fcitx/conf/fcitx-classic-ui.config",pwd->pw_name);
    printf("用户配置文件路径为：%s\n",usr_conffile);
   //Temporarily remove the update function
   //set_dingshi_thread();
    int test_flag = CheckUserConffile(usr_conffile);
    if (test_flag == 1)
	system("sudo change_xkb");

    FcitxConfigFileDesc* configDesc = GetClassicUIDesc();
    if (configDesc == NULL)
        return false;
    FILE *fp;
    fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-classic-ui.config", "r", NULL);
    if (!fp) {
        if (errno == ENOENT)
            SaveClassicUIConfig(classicui);
    }

    FcitxConfigFile *cfile = FcitxConfigParseConfigFileFp(fp, configDesc);

    FcitxClassicUIConfigBind(classicui, cfile, configDesc);
    FcitxConfigBindSync(&classicui->gconfig);

    if (fp)
        fclose(fp);
    return true;
}

void SaveClassicUIConfig(FcitxClassicUI *classicui)
{
    FcitxConfigFileDesc* configDesc = GetClassicUIDesc();
    FILE *fp = FcitxXDGGetFileUserWithPrefix("conf", "fcitx-classic-ui.config", "w", NULL);
    FcitxConfigSaveConfigFileFp(fp, &classicui->gconfig, configDesc);
    if (fp)
        fclose(fp);
}

boolean IsInRspArea(int x0, int y0, FcitxClassicUIStatus* status)
{
    return FcitxUIIsInBox(x0, y0, status->x, status->y, status->w, status->h);
}

boolean
ClassicUIMouseClick(FcitxClassicUI* classicui, Window window, int *x, int *y)
{
    boolean bMoved = false;
    FcitxX11MouseClick(classicui->owner, &window, x, y, &bMoved);
    return bMoved;
}

void ClassicUIOnTriggerOn(void* arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    MainWindowShow(classicui->mainWindow);
    TrayWindowDraw(classicui->trayWindow);
}

void ClassicUIOnTriggerOff(void* arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    MainWindowShow(classicui->mainWindow);
    TrayWindowDraw(classicui->trayWindow);
}

static void UpdateMainMenu(FcitxUIMenu* menu)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) menu->priv;
    FcitxInstance* instance = classicui->owner;
    FcitxMenuClear(menu);

    //FcitxMenuAddMenuItem(menu, _("Online Help"), MENUTYPE_SIMPLE, NULL);
   // FcitxMenuAddMenuItem(menu, NULL, MENUTYPE_DIVLINE, NULL);
    boolean flag = false;

    FcitxUIStatus* status;
    UT_array* uistats = FcitxInstanceGetUIStats(instance);
    for (status = (FcitxUIStatus*) utarray_front(uistats);
            status != NULL;
            status = (FcitxUIStatus*) utarray_next(uistats, status)
        ) {
        FcitxClassicUIStatus* privstat =  GetPrivateStatus(status);
        if (privstat == NULL || !status->visible)
            continue;

        flag = true;
        FcitxMenuAddMenuItemWithData(menu, status->shortDescription, MENUTYPE_SIMPLE, NULL, strdup(status->name));
    }

    FcitxUIComplexStatus* compstatus;
    UT_array* uicompstats = FcitxInstanceGetUIComplexStats(instance);
    for (compstatus = (FcitxUIComplexStatus*) utarray_front(uicompstats);
            compstatus != NULL;
            compstatus = (FcitxUIComplexStatus*) utarray_next(uicompstats, compstatus)
        ) {
        FcitxClassicUIStatus* privstat =  GetPrivateStatus(compstatus);
        if (privstat == NULL || !compstatus->visible)
            continue;
        if (FcitxUIGetMenuByStatusName(instance, compstatus->name))
            continue;

        flag = true;
        FcitxMenuAddMenuItemWithData(menu, compstatus->shortDescription, MENUTYPE_SIMPLE, NULL, strdup(compstatus->name));
    }

    if (flag)
        FcitxMenuAddMenuItem(menu, NULL, MENUTYPE_DIVLINE, NULL);

    FcitxUIMenu **menupp;
    UT_array* uimenus = FcitxInstanceGetUIMenus(instance);
    for (menupp = (FcitxUIMenu **) utarray_front(uimenus);
            menupp != NULL;
            menupp = (FcitxUIMenu **) utarray_next(uimenus, menupp)
        ) {
        FcitxUIMenu * menup = *menupp;
        if (menup->isSubMenu)
            continue;

        if (!menup->visible)
            continue;

        if (menup->candStatusBind) {
            FcitxUIComplexStatus* compStatus = FcitxUIGetComplexStatusByName(instance, menup->candStatusBind);
            if (compStatus) {
                if (!compStatus->visible)
                    continue;
            }
        }

        FcitxMenuAddMenuItem(menu, menup->name, MENUTYPE_SUBMENU, menup);
    }
    FcitxMenuAddMenuItem(menu, NULL, MENUTYPE_DIVLINE, NULL);
    //FcitxMenuAddMenuItem(menu, _("Add Umode Lexicon"), MENUTYPE_SIMPLE, NULL);
    FcitxMenuAddMenuItem(menu, _("Configure Current Input Method"), MENUTYPE_SIMPLE, NULL);
    FcitxMenuAddMenuItem(menu, _("Configure"), MENUTYPE_SIMPLE, NULL);
    FcitxMenuAddMenuItem(menu, _("FeedBack"), MENUTYPE_SIMPLE, NULL);
    if((access("/usr/share/fcitx/personal-center/fcitx_user",F_OK))!=-1)
    {  
    	FcitxMenuAddMenuItem(menu, _("Personal Center"), MENUTYPE_SIMPLE, NULL);
    }
   // Temporarily remove the update function
   // FcitxMenuAddMenuItem(menu, _("Upgrade"), MENUTYPE_SIMPLE, NULL);
    FcitxMenuAddMenuItem(menu, _("Restart"), MENUTYPE_SIMPLE, NULL);
//    FcitxMenuAddMenuItem(menu, _("Exit"), MENUTYPE_SIMPLE, NULL);
}

static void UpdateleftMainMenu(FcitxUIMenu* menu)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) menu->priv;
    FcitxInstance* instance = classicui->owner;
    //FcitxInstance* instance = (FcitxInstance*) menu->priv;
    FcitxMenuClear(menu);

    FcitxUIMenu **menupp;

    FcitxIM* pim;
    UT_array* imes = &instance->imes;
    for (pim = (FcitxIM *) utarray_front(imes);
            pim != NULL;
            pim = (FcitxIM *) utarray_next(imes, pim))
    {
	    printf("33333333333 %s\n",pim->strName);
     	    FcitxMenuAddMenuItem(menu, pim->strName, MENUTYPE_SIMPLE, NULL);
    }
   
}


int PersonalCenterOnOrNot()
{
    FILE *fp;
    if ((fp = popen("ps -A |grep im_personal_cen| awk '{printf $1}'", "r")) == NULL) 
    {
        perror("popen failed");
        return -1;
    }
    else
    {
	   char buf[256];
	   if (fgets(buf, 255, fp) != NULL) 
	    {
	    	printf("%s", buf);	      
	    }
	   else
	    {
	    	popen("/usr/share/fcitx/personal-center/im_personal_center","r");  
	    }
   	   if (pclose(fp) == -1)
            {
	     perror("pclose failed");
	     return -2;
	    }
    }

    return 0;
}
/********Upgrade Function*********/

int updatewindow()
{
   int source_list=0;
   char t[256];
   char one_line[1024];
    FILE *fin;FILE *source_list_ip;

        source_list_ip=fopen("/usr/share/fcitx/update/source_list_fcitx_ip","r");
        fin=fopen("/etc/apt/sources.list","r");
        fgets(t,255,source_list_ip);
        while(fgets(one_line,1023,fin)!=NULL)
        {
           if (strcmp(one_line,t)==0){
                   source_list = 1;
                   break;
             }
           memset(one_line,0,sizeof(one_line));
              } 
              printf("source_list-----%d=",source_list);
              fclose(fin);
              fclose(source_list_ip);
              return source_list;


}

void  SetsourcelistFcitxThread()
{
          system("sudo /usr/share/fcitx/update/source_list_fcitx");
}

void setfcitx_updatesourcelist_thread()
{
        GThread *g_thread;
        g_thread = g_thread_new("setsourcelistFcitx",SetsourcelistFcitxThread,NULL);

}



void  SetsourcelistUpdateThread()
{
   system("sudo /usr/share/fcitx/update/sourcelistupdate");
}

void set_sourcelistUpdate_thread()
{
          GThread *g_thread;
      g_thread = g_thread_new("setsourcelistupdate",SetsourcelistUpdateThread,NULL);

}

boolean IMMenuAction(FcitxUIMenu *menu, int index)
{
	
   	FcitxClassicUI* classicui = (FcitxClassicUI*) menu->priv;
    	FcitxInstance* instance = classicui->owner;
	FcitxInstanceSwitchIMByIndex(instance, index);
	return true;
}

boolean MainMenuAction(FcitxUIMenu* menu, int index)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) menu->priv;
    FcitxInstance* instance = classicui->owner;
    int length = utarray_len(&menu->shell);
/*    if (index == 0) {
        char* args[] = {
            "xdg-open",
            _("http://fcitx-im.org/"),
            0
        };
        fcitx_utils_start_process(args);*/
//    } else if (index == length - 1) { /* Exit */
//        FcitxInstanceEnd(classicui->owner);
/*    } else*/
    if((access("/usr/share/fcitx/personal-center/fcitx_user",F_OK))!=-1)
    { 
    	if (index == length - 1) { /* Restart */
		system("p_id=`ps  -aux | grep fcitx-config-gtk3 | grep -v grep |awk '{print $2}'`;kill $p_id");
        	fcitx_utils_launch_restart();
    	}
	else if (index == length - 3) { /* Feedback */
		popen("/usr/share/fcitx/typewriting-feedback/typewriting-feedback","r");
	} 
	else if (index == length - 2) { /* personal center */
		PersonalCenterOnOrNot();
	}
	else if (index == length - 4) { /* Configuration */
	        fcitx_utils_launch_configure_tool();
        }
        else if (index == length - 5) { /* Configuration */
	        FcitxIM* im = FcitxInstanceGetCurrentIM(classicui->owner);
		if (im && im->owner) {
			fcitx_utils_launch_configure_tool_for_addon(im->uniqueName);
		}
		else {
			fcitx_utils_launch_configure_tool();
		}
	} 

#if 0
	//Temporarily remove the update function

       	else if (index == length - 2) { /* Upgrade */
//		system("zenity --info");
                if(updatewindow()==0)
                     { setfcitx_updatesourcelist_thread();}
                   else
                  {
                        set_sourcelistUpdate_thread();

                  }
 
		Update_Fcitx();
    	} else if (index == length - 4) { /* Feedback */
        	popen("/usr/share/fcitx/typewriting-feedback/typewriting-feedback","r");
    	} else if (index == length - 3) { /* personal center */
        	PersonalCenterOnOrNot();
    	} else if (index == length - 5) { /* Configuration */
        	fcitx_utils_launch_configure_tool();
    	} else if (index == length - 6) { /* Configuration */
        	FcitxIM* im = FcitxInstanceGetCurrentIM(classicui->owner);
        	if (im && im->owner) {
            		fcitx_utils_launch_configure_tool_for_addon(im->uniqueName);
        	}
        	else {
            		fcitx_utils_launch_configure_tool();
        	}
    	} 
	else if (index == length - 7) { /* chaizi */
		popen("/usr/share/fcitx/umode/umodeLexicon","r");
	}
#endif

	else {
        	FcitxMenuItem* item = (FcitxMenuItem*) utarray_eltptr(&menu->shell, index);
        	if (item && item->type == MENUTYPE_SIMPLE && item->data) {
            		const char* name = item->data;
            		FcitxUIUpdateStatus(instance, name);
        	}
    	}
    }
    else
    {
 	if (index == length - 1) 
        { /* Restart */
		system("p_id=`ps  -aux | grep fcitx-config-gtk3 | grep -v grep |awk '{print $2}'`;kill $p_id");
        	fcitx_utils_launch_restart();
    	}
	else if (index == length - 2) { /* Feedback */
		popen("/usr/share/fcitx/typewriting-feedback/typewriting-feedback","r");
	}
        else if (index == length - 3) { /* Configuration */
	        fcitx_utils_launch_configure_tool();
	}
        else if (index == length - 4) { /* Configuration */
	        FcitxIM* im = FcitxInstanceGetCurrentIM(classicui->owner);
		if (im && im->owner) {
		    fcitx_utils_launch_configure_tool_for_addon(im->uniqueName);
		}
		else {
		    fcitx_utils_launch_configure_tool();
		}
	}


#if 0
        //Temporarily remove the update function
	//
	else if (index == length - 2) 

         { /* Upgrade */
                 if(updatewindow()==0)
                     { setfcitx_updatesourcelist_thread();}
                   else
                  {
                        set_sourcelistUpdate_thread();

                  }
 
                 Update_Fcitx();
    	} else if (index == length - 3) { /* Feedback */
        	popen("/usr/share/fcitx/typewriting-feedback/typewriting-feedback","r");
    	} else if (index == length - 4) { /* Configuration */
        	fcitx_utils_launch_configure_tool();
    	} else if (index == length - 5) { /* Configuration */
        	FcitxIM* im = FcitxInstanceGetCurrentIM(classicui->owner);
        	if (im && im->owner) {
            		fcitx_utils_launch_configure_tool_for_addon(im->uniqueName);
        	}
        	else {
            		fcitx_utils_launch_configure_tool();
        	}
    	}
	else if (index == length - 6) { /* chaizi */

		popen("/usr/share/fcitx/umode/umodeLexicon","r");
	}
#endif
       	else {
        	FcitxMenuItem* item = (FcitxMenuItem*) utarray_eltptr(&menu->shell, index);
        	if (item && item->type == MENUTYPE_SIMPLE && item->data) {
            		const char* name = item->data;
            		FcitxUIUpdateStatus(instance, name);
        	}
    	}
    }
    return true;
}

Visual * ClassicUIFindARGBVisual(FcitxClassicUI* classicui)
{
    return FcitxX11FindARGBVisual(classicui->owner);
}

void ClassicUIMainWindowSizeHint(void* arg, int* x, int* y, int* w, int* h)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    if (x) {
        *x = classicui->iMainWindowOffsetX;
    }
    if (y) {
        *y = classicui->iMainWindowOffsetY;
    }

    XWindowAttributes attr;
    XGetWindowAttributes(classicui->dpy, classicui->mainWindow->parent.wId, &attr);
    if (w) {
        *w = attr.width;
    }
    if (h) {
        *h = attr.height;
    }

}

void ReloadConfigClassicUI(void* arg)
{
    FcitxClassicUI* classicui = (FcitxClassicUI*) arg;
    LoadClassicUIConfig(classicui);
    DisplaySkin(classicui, classicui->skinType);
}

boolean WindowIsVisable(Display* dpy, Window window)
{
    XWindowAttributes attr;
    XGetWindowAttributes(dpy, window, &attr);
    return attr.map_state == IsViewable;
}

boolean EnlargeCairoSurface(cairo_surface_t** sur, int w, int h)
{
    int ow = cairo_image_surface_get_width(*sur);
    int oh = cairo_image_surface_get_height(*sur);

    if (ow >= w && oh >= h)
        return false;

    while (ow < w) {
        ow *= 2;
    }

    while (oh < h) {
        oh *= 2;
    }

    cairo_surface_destroy(*sur);
    *sur = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, ow, oh);
    return true;
}

void ResizeSurface(cairo_surface_t** surface, int w, int h)
{
    int ow = cairo_image_surface_get_width(*surface);
    int oh = cairo_image_surface_get_height(*surface);

    if ((ow == w && oh == h) || w == 0 || h == 0 || ow == 0 || oh == 0)
        return;

    double scalex = (double)w / ow;
    double scaley = (double)h / oh;
    double scale = (scalex > scaley) ? scaley : scalex;

    int nw = ow * scale;
    int nh = oh * scale;

    cairo_surface_t* newsurface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w, h);
    cairo_t* c = cairo_create(newsurface);
    cairo_set_operator(c, CAIRO_OPERATOR_SOURCE);
    cairo_set_source_rgba(c ,1, 1, 1, 0.0);
    cairo_paint(c);
    cairo_translate(c, (w - nw) / 2.0 , (h - nh) / 2.0);
    cairo_scale(c, scale, scale);
    cairo_set_source_surface(c, *surface, 0, 0);
    cairo_rectangle(c, 0, 0, ow, oh);
    cairo_clip(c);
    cairo_paint(c);
    cairo_destroy(c);

    cairo_surface_destroy(*surface);

    *surface = newsurface;
}

#include "fcitx-classic-ui-addfunctions.h"
