#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QDebug>
#include <QFile>
#include <phonogramDialog.h>
#include <QNetworkRequest>
#include <QNetworkAccessManager>
#include <QMessageBox>
#include <QTextCodec>

//#define UMODE_LEXICON_SERVER "http://192.168.1.105:8080/im_service/umodeLexicon.action"
#define FCITX_UMODE_SERVER_PATH "/usr/share/fcitx/umode/umode_server.txt"


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{

    ui->setupUi(this);
    phonogramNum = 0;
    //this->setWindowTitle(tr("Add Lexicon"));
    this->setWindowIcon(QIcon("/usr/share/fcitx/skin/dark/inactive.png"));
    QTextCodec *codec = QTextCodec::codecForName("utf-8");
    setWindowTitle(codec->toUnicode("词库添加"));

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::receivePhonogramData(QString data)
{
    QString currentInputValue = ui->PhonogramLineEdit->text();
    phonogramNum = data.length() + phonogramNum;
    qDebug()<<"phonogramNum" << phonogramNum <<  data.length();
    qDebug() << " in receive " << data;
    ui->PhonogramLineEdit->setText(currentInputValue + data);     //获取传递过来的数据
}
QString MainWindow::getChinese(QString &str)
{
    //    QString str;
    QString chineseStr;
    int nCount = str.count();
    for(int i = 0 ; i < nCount ; i++)
    {
        QChar cha = str.at(i);
        ushort uni = cha.unicode();
        if(uni >= 0x4E00 && uni <= 0x9FA5)
        {
            //这个字符是中文
            qDebug()<<uni;
            chineseStr.append(uni);
            //            qDebug("%s",uni);
        }
    }
    return chineseStr;  //最后返回的这个字符串就是中文字符串

}
void MainWindow::on_pushButton_clicked()
{
    QString phonograml = ui->PhonogramLineEdit->text();

    QString spell = ui->spellLineEdit->text();

    QString candidate = ui->candidateWordLineEdit->text();


    qDebug()<< "phonograml" << phonograml <<endl ;

    if(phonograml.isEmpty() || spell.isEmpty() || candidate.isEmpty())
    {
        QMessageBox::information(NULL, tr("Warning"), tr("The input content can not be empty"), tr("YES"));
        return;

    }

    if( phonogramNum == 0)
    {
        phonogramNum = 0;
        ui->PhonogramLineEdit->setText("");
        QMessageBox::information(NULL, tr("Warning"), tr("The pinyin must contain the sound mark"), tr("YES"));
        return;

    }
    if(phonogramNum > 1)
    {
        phonogramNum = 0;
        ui->PhonogramLineEdit->setText("");
        QMessageBox::information(NULL, tr("Warning"), tr("The sound mark must be unique"), tr("YES"));
        return;

    }
    QString spellchineseContent = getChinese(spell);
    if(! spellchineseContent.isEmpty() || spellchineseContent.length() > 0 )
    {
        QMessageBox::information(NULL, tr("Warning"), tr("Please enter the spelling of the split word that does not contain Chinese characters"), tr("YES"));
        return;
    }

    QString chineseContent = getChinese(candidate);
    if(chineseContent.isEmpty() || chineseContent.length() != candidate.length() )
    {

        QMessageBox::information(NULL, tr("Warning"), tr("The candidate words can only be Chinese"), tr("YES"));
        return;
    }
    if(candidate.length() > 1)
    {

        QMessageBox::information(NULL, tr("Warning"), tr("The candidate words must be unique"), tr("YES"));
        return;
    }

    QNetworkRequest request;
    QFile file(FCITX_UMODE_SERVER_PATH);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug()<<"Can't open the file!"<<endl;
    }

    QByteArray line = file.readLine();
    QString serverPath(line);
    serverPath = serverPath.remove("\n");
    qDebug()<< serverPath;

    QString url_info = serverPath +"/im_service/umodeLexicon.action?spell=" + spell + "&candidate=" + candidate + "&phonogram=" + phonograml ;
    qDebug()<< "url_info "<<url_info;
    request.setUrl(QUrl(url_info));

    QNetworkAccessManager *m_accessManager;
    m_accessManager = new QNetworkAccessManager();
    m_accessManager->get(request);

    QObject::connect(m_accessManager, SIGNAL(finished(QNetworkReply*)), this, SLOT(finishedSlot(QNetworkReply*)));



}
void MainWindow::finishedSlot(QNetworkReply *reply)
{
     if (reply->error() == QNetworkReply::NoError)
     {
         QByteArray bytes = reply->readAll();
         qDebug()<<bytes;

         QMessageBox::information(NULL, tr("Info"), tr("add lexicon success"), tr("YES"));


     }
     else
     {
         qDebug()<<"handle errors here";
         QVariant statusCodeV = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
         //statusCodeV是HTTP服务器的相应码，reply->error()是Qt定义的错误码，可以参考QT的文档
         qDebug( "found error ....code: %d %d\n", statusCodeV.toInt(), (int)reply->error());
         qDebug(qPrintable(reply->errorString()));

         QMessageBox::information(NULL, tr("Warning"), tr("add lexicon failed"), tr("YES"));
     }
     reply->deleteLater();
}
void MainWindow::on_addPhonogramButton_clicked()
{

    addPhonogramDialog = new Dialog();
    addPhonogramDialog->show();
    connect(addPhonogramDialog, SIGNAL(sendPhonogramData(QString)) , this , SLOT(receivePhonogramData(QString)) );


}
